Package: scTreeViz
Type: Package
Title: R/Bioconductor package to interactively explore and visualize
        single cell RNA-seq datasets with hierarhical annotations
Version: 1.19.0
Authors@R: c( 
          person("Jayaram", "Kancherla", email="jayaram.kancherla@gmail.com", role=c("aut", "cre")),
          person("Hector", "Corrada Bravo", email="hcorrada@gmail.com", role=c("aut")),
          person("Kazi Tasnim", "Zinat", email="kzintas@umd.edu", role=c("aut")),
          person("Stephanie", "Hicks", email="shicks19@jhu.edu", role=c("aut"))
          )
Description: scTreeViz provides classes to support interactive data
        aggregation and visualization of single cell RNA-seq datasets
        with hierarchies for e.g. cell clusters at different
        resolutions. The `TreeIndex` class provides methods to manage
        hierarchy and split the tree at a given resolution or across
        resolutions. The `TreeViz` class extends `SummarizedExperiment`
        and can performs quick aggregations on the count matrix defined
        by clusters.
License: Artistic-2.0
Depends: R (>= 4.0), methods, epivizr, SummarizedExperiment
Imports: data.table, S4Vectors, digest, Matrix, Rtsne, httr, igraph,
        clustree, scran, sys, epivizrData, epivizrServer, ggraph,
        scater, Seurat, SingleCellExperiment, ggplot2, stats, utils
Suggests: knitr, BiocStyle, testthat, SC3, scRNAseq, rmarkdown, msd16s,
        metagenomeSeq, epivizrStandalone, GenomeInfoDb
VignetteBuilder: knitr
biocViews: Visualization, Infrastructure, GUI, SingleCell
Encoding: UTF-8
Collate: 'TreeIndex-class.R' 'TreeIndex-methods.R' 'TreeViz-class.R'
        'TreeViz-methods.R' 'startTreeViz.R' 'TreeVizApp-class.R'
        'EpivizTreeData-class.R' 'helper-functions.R'
        'ClusterHierarchy-class.R'
RoxygenNote: 7.1.1
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev libfribidi-dev libglpk-dev make
        libharfbuzz-dev libbz2-dev libicu-dev libjpeg-dev liblzma-dev
        libpng-dev libtiff-dev libuv1-dev libwebp-dev libxml2-dev
        libssl-dev python3 xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:57:06 UTC
RemoteUrl: https://github.com/bioc/scTreeViz
RemoteRef: HEAD
RemoteSha: dd15f611114a451159c57445cab5d61496dbf137
NeedsCompilation: no
Packaged: 2026-05-30 09:55:02 UTC; root
Author: Jayaram Kancherla [aut, cre],
  Hector Corrada Bravo [aut],
  Kazi Tasnim Zinat [aut],
  Stephanie Hicks [aut]
Maintainer: Jayaram Kancherla <jayaram.kancherla@gmail.com>
Built: R 4.6.0; ; 2026-05-30 15:23:10 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ClusterHierarchy)
export(TreeIndex)
export(TreeViz)
export(aggregateTree)
export(createFromSCE)
export(createFromSeurat)
export(createTreeViz)
export(startTreeviz)
exportClasses(EpivizTreeData)
exportClasses(TreeViz)
exportClasses(TreeVizApp)
exportMethods(aggregateTree)
exportMethods(getNodes)
exportMethods(plot)
exportMethods(show)
exportMethods(splitAt)
import(Rtsne)
import(SingleCellExperiment)
import(SummarizedExperiment)
import(clustree)
import(data.table, except=shift)
import(digest)
import(epivizr)
import(ggraph)
import(httr)
import(igraph)
import(methods)
import(sys)
importClassesFrom(epivizr,EpivizApp)
importClassesFrom(epivizrData,EpivizData)
importClassesFrom(epivizrData,EpivizDataMgr)
importClassesFrom(epivizrData,EpivizMeasurement)
importClassesFrom(epivizrData,SparseEpivizMeasurement)
importClassesFrom(epivizrServer,EpivizServer)
importFrom(Matrix,colSums)
importFrom(Matrix,rowSums)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,metadata)
importFrom(Seurat,GetAssayData)
importFrom(Seurat,Reductions)
importFrom(Seurat,as.SingleCellExperiment)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,melt)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,setorderv)
importFrom(ggplot2,aes)
importFrom(igraph,as_long_data_frame)
importFrom(igraph,graph_from_data_frame)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(scater,calculateTSNE)
importFrom(scran,getTopHVGs)
importFrom(scran,modelGeneVar)
importFrom(stats,na.omit)
importFrom(utils,str)
importMethodsFrom(epivizrData,register)

# scTreeViz
This package provides methods to interactively explore and visualize datasets with hierarchies. eg. single cells datasets with hierarchy over cells at different resolutions. 

For a walk through of various features of the app, checkout the vignette at 

https://hcbravolab.github.io/scTreeViz/Explore-PBMC.html

## Installation and requirements

`scTreeViz` currently requires a development version of `epivizrData` which supports Sparse Measurements. We plan to push this to `bioc` soon. 

To install `scTreeViz`, 

```{r}
BiocManager::install("epiviz/epivizrData")
BiocManager::install("HCBravoLab/scTreeViz")
```
***Note: you can also use devtools to install the package from github***

## Usage

To try out and explore the various features of the package, we provide the PBMC 3K dataset and the vignette `Explore-PBMC` walks you through the process. 

To checkout all the vignettes

```{r}
library(scTreeViz)
browseVignettes("scTreeViz")
```

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----load-packages, message=FALSE, warning=FALSE------------------------------
library(scTreeViz)
library(Seurat)
library(SC3)
library(scran)
library(scater)
library(clustree)
library(igraph)
library(scRNAseq)

## ----results='hide', warning=FALSE, error=FALSE, message=FALSE----------------
# load dataset
sce<- ZeiselBrainData()
# Normalization
sce <- logNormCounts(sce)
# calculate umap and tsne
sce <- runUMAP(sce)
sce<- runTSNE(sce)
sce<- runPCA(sce)

## ----warning=FALSE, error=FALSE, message=FALSE--------------------------------
treeViz <- createFromSCE(sce, reduced_dim = c("UMAP","PCA","TSNE"))
plot(treeViz)

## ----eval=FALSE, warning=FALSE, error=FALSE, message=FALSE--------------------
# # Forming clusters
# set.seed(1000)
# for (i in  seq(10)) {
#   clust.kmeans <- kmeans(reducedDim(sce, "TSNE"), centers = i)
#   sce[[paste0("clust", i)]] <- factor(clust.kmeans$cluster)
# }
# 
# treeViz<- createFromSCE(sce, check_coldata = TRUE, col_regex = "clust")
# plot(treeViz)

## ----echo=TRUE, results='hide', warning=FALSE, error=FALSE, message=FALSE-----
data(pbmc_small)
pbmc <- pbmc_small

## ----echo=TRUE, results='hide', warning=FALSE, error=FALSE, message=FALSE-----
pbmc[["percent.mt"]] <- PercentageFeatureSet(pbmc, pattern = "^MT-")
pbmc <- NormalizeData(pbmc)
all.genes <- rownames(pbmc)
pbmc <- ScaleData(pbmc, vars.to.regress = "percent.mt")
pbmc <- FindVariableFeatures(object = pbmc)
pbmc <- RunPCA(pbmc, features = VariableFeatures(object = pbmc))
pbmc <- FindNeighbors(pbmc, dims = 1:10)
pbmc <- FindClusters(pbmc, resolution = c(0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0), print.output = 0, save.SNN = TRUE)
pbmc

## ----echo=TRUE, results='hide', warning=FALSE, error=FALSE, message=FALSE-----
# pbmc<- RunTSNE(pbmc)
pbmc<- RunUMAP(pbmc, dims=1:3)
Reductions(pbmc)

## ----echo=TRUE,  warning=FALSE, error=FALSE, message=FALSE--------------------
treeViz<- createFromSeurat(pbmc, check_metadata = TRUE, reduced_dim = c("umap","pca","tsne"))
plot(treeViz)

## ----results='hide', warning=FALSE, error=FALSE, message=FALSE----------------
n=64
# create a hierarchy
df<- data.frame(cluster0=rep(1,n))
for(i in seq(1,5)){
  df[[paste0("cluster",i)]]<- rep(seq(1:(2**i)),each=ceiling(n/(2**i)),len=n)
}

# generate a count matrix
counts <- matrix(rpois(6400, lambda = 10), ncol=n, nrow=100)
colnames(counts)<- seq(1:64)
# create a `TreeViz` object
treeViz <- createTreeViz(df, counts)
plot(treeViz)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# app <- startTreeviz(treeViz, top_genes = 500)

## ----eval=FALSE---------------------------------------------------------------
# app$plotGene(gene="AIF1")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# app$stop_app()

## -----------------------------------------------------------------------------
sessionInfo()

#' Class managing connection to metaviz application.
#' 
#' @importClassesFrom epivizr EpivizApp
#' @importClassesFrom epivizrServer EpivizServer
#' @importClassesFrom epivizrData EpivizDataMgr EpivizMeasurement EpivizData
#' @exportClass TreeVizApp
TreeVizApp <- setRefClass("TreeVizApp",
  contains = "EpivizApp",
  fields=list(
    .url_parms="list",
    .browser_fun="function",
    server="EpivizServer",
    data_mgr="EpivizDataMgr",
    chart_mgr="EpivizChartMgr"
  ),
  methods=list(
    initialize=function(.url_parms=.url_parms, .browser_fun=.browser_fun,
                        server=server, data_mgr=data_mgr, chart_mgr=chart_mgr) {
      callSuper(.url_parms=.url_parms, .browser_fun=.browser_fun,
                server=server, data_mgr=data_mgr, chart_mgr=chart_mgr)
    }
  )
)

TreeVizApp$methods(
  plotGene = function(gene = NULL, datasource_name = "SCRNA_1") {
    " Plot a bar plot for a gene across cell types
    \\describe{
    \\item{gene}{gene to extract expression values}
    \\item{datasource_name}{object to extract from (automatically selected)}
    }
    "
    
    if (is.null(gene)) {
      stop("gene symbol must be provided")
    }
    
    obj <- .self$data_mgr$.find_datasource(datasource_name)
    if (is.null(obj)) {
      stop("cannot find datasource", datasource_name)
    }
    
    ms_list <-obj$get_measurements()
    subset_ms_list <- Filter(function(ms) ms@id == gene, ms_list)
    
    if (length(subset_ms_list) == 0) {
      stop("cannot find the gene in the dataset!")
    }
    
    .self$chart_mgr$visualize(chart_type = "DiversityScatterPlot",  measurements = subset_ms_list)
    .self$server$wait_to_clear_requests()
    .delay_requests(.self$server)
    
  },
  
  extract_SCE = function(cluster_name="treeviz_clusters", datasource_name = "SCRNA_1") {
    obj <- .self$data_mgr$.find_datasource(datasource_name)
    if (is.null(obj)) {
      stop("cannot find datasource", datasource_name)
    }

    sce<-obj$extract_SCE_epiviz(cluster_name)
  }
)

#' Data container for MRexperiment objects
#'
#' Used to serve hierarchical data (used in e.g., icicle plots and heatmaps).
#' @importClassesFrom epivizrData EpivizData
#' @importClassesFrom epivizrData SparseEpivizMeasurement
#' @rawNamespace import(data.table, except=shift)
#' @import digest
#' @import methods
#' @import httr
#' @import Rtsne
#' @exportClass EpivizTreeData
EpivizTreeData <- setRefClass("EpivizTreeData",
  contains = "EpivizData",
  fields = list(
    .levels = "ANY",
    .maxDepth = "numeric",
    .feature_order = "character",
    .minValue = "numeric",
    .maxValue = "numeric",
    .sampleAnnotation = "ANY",
    .nodeSelections = "ANY",
    .levelSelected = "ANY",
    .lastRootId = "character",
    .json_query = "ANY",
    .graph = "ANY",
    .treeIn = "character",
    .measurements = "ANY"
  ),
  
  methods=list(
    initialize=function(object, tree="row", columns=NULL, control=metavizControl(), feature_order=NULL, ...) {
      
      if(is.null(feature_order)) {
        if(tree == "row") {
          .self$.feature_order = colnames(rowData(object))
        } else {
          .self$.feature_order = colnames(colData(object))
        }
      } else {
        .self$.feature_order <- feature_order
      }
      
      .self$.levelSelected <- 3
      .self$.lastRootId <- "0-0"
      .self$.nodeSelections <- list()
      .self$.treeIn <- tree
      .self$.measurements <- NULL
      
      if(tree == "row") {
        .self$.graph <- rowData(object)
      } else {
        .self$.graph <- colData(object)
      }
      
      featureSelection = NULL
      
      if(!is.null(featureSelection)) {
        featureSelection <- featureSelection[which(names(featureSelection) != "NA")]
        featureSelection <- featureSelection[which(names(featureSelection) != "no_match")]
        
        if(tree == "row") {
          node_ids <- vapply(names(featureSelection), function(n) {
            as.character(rowData(object)$nodes_table[node_label==n,id])
          }, character(1))
        } else {
          node_ids <- vapply(names(featureSelection), function(n) {
            as.character(colData(object)$nodes_table[node_label==n,id])
          }, character(1))
        }
        
        temp_selections <- unname(featureSelection)
        names(temp_selections) <- node_ids
        .self$.nodeSelections = temp_selections
      }
      
      callSuper(object=object, ...)
    }
  )
)

# Epiviz Websockets Protocol
EpivizTreeData$methods(
  get_default_chart_type=function() {
    "epiviz.ui.charts.tree.Icicle"
  },
  
  get_measurements=function(top_genes=TRUE) {
    "Get all annotation info for all samples
    \\describe{
    \\item{chart_id_or_object}{An object of class \\code{EpivizChart} or an id for
    a chart loaded to the epiviz app.}
    }
    "
    
    if(top_genes) {
      
      if (!is.null(.self$.measurements)) {
        out <- .self$.measurements
      }
      else {
        genes <- metadata(.self$.object)$top_variable
        
        out <- lapply(genes, function(gene) {
          epivizrData:::SparseEpivizMeasurement(id=gene,
                                                datasourceId=.self$.id)
        })
        
        .self$.measurements <- out  
      }
      return(out)
    }
    
    if (.self$.treeIn == "row") {
      sample_table <- as.data.frame(colData(.self$.object))
      # .sampleAnnotation <- colData(.self$.object)
    } else {
      sample_table <- as.data.frame(rowData(.self$.object))
      # .sampleAnnotation <- rowData(.self$.object)
    }
    
    if (!is.null(.self$.measurements)) {
      out <- .self$.measurements
    }
    else {
      out <- lapply(rownames(sample_table), function(sample) {
        epivizrData:::SparseEpivizMeasurement(id=sample,
          datasourceId=.self$.id)
      })
      
      .self$.measurements <- out 
    }
    
    return(out)
  },
  
  row_to_dict=function(row){
    "Helper function to format each node entry for getHierarchy response
    \\describe{
    \\item{row}{Information for current node.}
    }
    "
    
    toRet = list()
    toRet['end'] = row['end']
    toRet['partition'] = "NA"
    toRet['leafIndex'] = row['leafIndex']
    toRet['nchildren'] = row['nchildren']
    toRet['label'] = row['label']
    toRet['name'] = row['label']
    toRet['start'] = row['start']
    toRet['depth'] = row['depth']
    toRet['globalDepth'] = row['depth']
    toRet['nleaves'] = row['nleaves']
    toRet['parentId'] = row['parentId']
    toRet['order'] = row['order']
    toRet['id'] = row['id']
    toRet['selectionType'] = 1
    toRet['taxonomy'] = row['taxonomy']
    toRet['size'] = 1
    toRet['children'] = NULL
    return(toRet)
  },
  
  df_to_tree=function(root, df){
    "Helper function to recursively build nested response for getHierarchy
    \\describe{
    \\item{root}{Root of subtree}
    \\item{df}{data.frame containing children to process}
    }
    "
    
    if(nrow(df) == 0) {
      root$children = NULL
      return(root)
    }
    
    children = df[which(df['parentId'] == as.character(unlist(root['id']))),]
    
    if(length(children) == 0){
      root$children = NULL
      return(root)
    }
    
    otherChildren = df[which(df['parentId'] != as.character(unlist(root['id']))),]
    
    children = children[order(children['order']),]
    
    if(nrow(children) > 0){
      for(row_index in seq_len(nrow(children))){
        childDict = row_to_dict(children[row_index,])
        subDict = df_to_tree(childDict, otherChildren)
        
        if(!is.null(subDict)){
          root$children[[row_index]] = subDict
        }
        else {
          root$children = NULL
        }
      }
    }
    return(root)
  },
  
  getHierarchy=function(nodeId = NULL) {
    "Retrieve feature hierarchy information for subtree with specified root
    \\describe{
    \\item{nodeId}{Feature identifier with level info}
    }
    "
    
    # getHierarchy can be called with NULL from App
    if(is.null(nodeId) || nodeId == ""){
      nodeId <- .self$.lastRootId
    }
    .self$.lastRootId <- nodeId
    root <- nodeId
    
    #Split the node id to get level and index
    split_res <- strsplit(nodeId, "-")[[1]]
    level <- as.integer(split_res[1])+1
    index <- which(.self$.graph@node_ids_table[,level, with=FALSE] == nodeId)
    
    graph_tree <- .self$.graph@hierarchy_tree[,-which(colnames(.self$.graph@hierarchy_tree) == "otu_index")]
    
    label <- as.character(unique(graph_tree[,level][index]))
    taxonomy <- colnames(graph_tree)[level]
    
    if(length(.self$.graph@feature_order) >= level+3){
      last_level_of_subtree <- level+3
    } else{
      last_level_of_subtree <- length(.self$.graph@feature_order)
    }
    
    hierarchy_slice <- unique(.self$.graph@node_ids_table[get(taxonomy)==nodeId, (level+1):last_level_of_subtree])
    
    nodes_of_subtree <- lapply(seq_len(length((level+1):last_level_of_subtree)), function(i) {
      unname(unlist(unique(hierarchy_slice[,i, with=FALSE])))
    })
    
    nodes_of_subtree <- unlist(nodes_of_subtree)
    
    if(level == 0 || nodeId == "0-0"){
      nodesToRet <- c(root, unlist(nodes_of_subtree))
    } else{
      parent_of_root_taxonomy <- colnames(graph_tree)[(level-1)]
      parent_of_root <- unique(.self$.graph@node_ids_table[get(taxonomy)==nodeId, get(parent_of_root_taxonomy)])
      nodesToRet <- c(parent_of_root,root, unlist(nodes_of_subtree))
    }
    
    num_rows <- length(nodesToRet)
    
    starts <- rep(1, num_rows)
    labels <- rep(1, num_rows)
    leafIndexes <- rep(1, num_rows)
    parentIds <- rep(1, num_rows)
    depths <- rep(0, num_rows)
    partitions <- rep(1, num_rows)
    ends <- rep(1, num_rows)
    ids <- rep(1, num_rows)
    nchildrens <- rep(1, num_rows)
    taxonomys <- rep(1, num_rows)
    nleaves <- rep(1, num_rows)
    orders <- rep(1, num_rows)
    
    leaf_ordering_table <- as.data.table(.self$.graph@hierarchy_tree[,c(.self$.graph@feature_order[length(.self$.graph@feature_order)], "otu_index")])
    setnames(leaf_ordering_table, c("leaf", "otu_index"))
    
    lineage_DF <- as.data.frame(.self$.graph@node_ids_table)
    lineage_table <- .self$.graph@node_ids_table
    lineage_DF[,.self$.graph@feature_order[1]] <- lineage_table[,get(.self$.graph@feature_order[1])]
    
    for(i in seq(2,length(.self$.graph@feature_order))){
      lineage_DF[,.self$.graph@feature_order[i]] <- paste(lineage_DF[,.self$.graph@feature_order[i-1]], lineage_table[,get(.self$.graph@feature_order[i])], sep=",")
    }
    lineage_DT <- as.data.table(lineage_DF)
    
    ncols <- ncol(lineage_DT)
    lineage_DT_long <- melt(lineage_DT, id.vars = c("otu_index"),
                            measure.vars = c(colnames(lineage_DT)[seq_len(ncols-1)]),
                            variable.name = "taxonomy", variable.factor = FALSE)
    
    setnames(lineage_DT_long, c("otu_index", "taxonomy", "lineage"))
    
    temp_nodes_table <- merge(.self$.graph@nodes_table, lineage_DT_long, by="lineage")
    temp_nodes_table <- temp_nodes_table[, otu_index:=as.integer(otu_index)]
    
    
    for(i in seq_len(num_rows)){
      if(as.integer(strsplit(nodesToRet[i], "-")[[1]][1]) == last_level_of_subtree){
        depths[i] = length(.self$.graph@feature_order)
        level = length(.self$.graph@feature_order)
        index <- which(.self$.graph@node_ids_table[,level,with=FALSE] == nodeId)
        
        label <- as.character(unique(graph_tree[,level][index]))
        labels[i] <- label
        
        partition <- "NA"
        partitions[i] <- partition
        
        otu_index_temp <- leaf_ordering_table[leaf == nodeId, otu_index]
        starts[i] <- otu_index_temp
        leafIndexes[i] <- otu_index_temp
        ends[i] <- otu_index_temp
        
        ids[i] <- nodeId
        
        taxonomy <- colnames(graph_tree)[level]
        taxonomys[i] <- taxonomy
        
        nchildrens[i] <- 0
        nleaves[i] <- 0
        
        orders[i] <- .self$.graph@nodes_table[get("id")==nodeId,get("order")][[1]]
      } else{
        nodeId <- nodesToRet[i]
        split_res <- strsplit(nodesToRet[i], "-")[[1]]
        depths[i] <- as.integer(split_res[1])
        level <- as.integer(split_res[1])+1
        
        index <- which(.self$.graph@node_ids_table[,level,with=FALSE] == nodeId)
        
        label <- as.character(unique(graph_tree[,level][index]))
        labels[i] <- label
        taxonomy <- colnames(graph_tree)[level]
        
        if(nodesToRet[i] != "0-0"){
          parentId_taxonomy <- colnames(graph_tree)[(level-1)]
          parentId <- unique(.self$.graph@node_ids_table[get(taxonomy)==nodesToRet[i], get(parentId_taxonomy)])[1]
          parentIds[i] <- parentId
        } else{
          parentIds[i] <- "NA"
        }
        
        partition <- "NA"
        partitions[i] <- partition
        
        if(nodesToRet[i] == "0-0"){
          leaf_indexes_temp <- temp_nodes_table[level == (length(.self$.graph@feature_order)-1), otu_index,]
        } else{
          ids_match <- .self$.graph@nodes_table[id == nodesToRet[i],]
          parents_match <- ids_match[parent == parentIds[i]]
          leaf_indexes_temp <- temp_nodes_table[lineage == parents_match[,lineage,], otu_index,]
        }
        if(length(leaf_indexes_temp) > 0){
          start <- min(leaf_indexes_temp)
        }    else{
          start <- nodesToRet[i]
        }
        
        if(length(leaf_indexes_temp) > 0){
          end <- max(leaf_indexes_temp)
        }    else{
          end <- nodesToRet[i]
        }
        
        starts[i] <- start
        
        leafIndex <- start
        leafIndexes[i] <- leafIndex
        ends[i] <- end
        
        id <- nodesToRet[i]
        ids[i] <- id
        
        taxonomy <- colnames(graph_tree)[level]
        taxonomys[i] <- taxonomy
        
        nchildren <- length(unique(.self$.graph@node_ids_table[get(taxonomy)==nodesToRet[i],][[as.integer(level)+1]]))
        nchildrens[i] <- nchildren[1]
        
        nleaves_temp <- length(unname(unlist(unique(.self$.graph@leaf_of_table[node_label==label, leaf]))))
        nleaves[i] <- nleaves_temp[1]
        
        if(nodesToRet[i] != "0-0"){
          orders[i] <- .self$.graph@nodes_table[get("id")==nodesToRet[i],get("order")][[1]]
        } else {
          orders[i] <- 1
        }
      }
    }
    ret_data_frame <- data.frame(start = starts, label = labels, leafIndex = leafIndexes, parentId = parentIds,
                                 depth = depths, partition = partitions, end = ends, id = ids, nchildren = nchildrens,
                                 taxonomy = taxonomys, nleaves = nleaves, order = orders)
    
    if(length(ret_data_frame) > 0){
      # convert columns to int
      ret_data_frame['start'] = as.numeric(unlist(ret_data_frame['start']))
      ret_data_frame['end'] = as.numeric(unlist(ret_data_frame['end']))
      ret_data_frame['order'] = as.numeric(unlist(ret_data_frame['order']))
      ret_data_frame['leafIndex'] = as.numeric(unlist(ret_data_frame['leafIndex']))
      ret_data_frame['nchildren'] = as.numeric(unlist(ret_data_frame['nchildren']))
      ret_data_frame['nleaves'] = as.numeric(unlist(ret_data_frame['nleaves']))
      ret_data_frame['depth'] = as.numeric(unlist(ret_data_frame['depth']))
      ret_data_frame['id'] = as.character(unlist(ret_data_frame['id']))
      
      
      root = ret_data_frame[1,]
      rest = ret_data_frame[-1,]
      rootDict = .self$row_to_dict(root)
      result = .self$df_to_tree(rootDict, rest)
      
      result[["rootTaxonomies"]] = .self$.graph@feature_order
      lineage = .self$.graph@nodes_table[get("id")==nodesToRet[1],get("lineage")][[1]]
      
      lineageLabel <- vapply(strsplit(lineage, ",")[[1]], function(str_id) {
        .self$.graph@nodes_table[get("id") == str_id, get("node_label")][[1]]
      }, character(1))
      
      result[["lineageLabel"]] = paste(lineageLabel, sep=", ")
      
      resultResp = list(nodeSelectionTypes = .self$.nodeSelections,
                        selectionLevel = .self$.levelSelected,
                        tree = result)
      
      return(resultResp)
    }
    
    return(ret_data_frame)
  },
  
  propagateHierarchyChanges=function(selection = NULL, order = NULL, selectedLevels = NULL, request_with_labels = FALSE) {
    "Update internal state for hierarchy
    \\describe{
    \\item{selection}{Node-id and selectionType pairs}
    \\item{order}{Ordering of features}
    \\item{selectedLevels}{Current aggregation level}
    \\item{request_with_labels}{For handling requests using fData entries from MRexperiment}
    }
    "
    
    if(request_with_labels && !is.null(selection)){
      selection_ids <- vapply(names(selection), function(i){
        .self$.graph@nodes_table[node_label==i,id]
      }, character(1))
      names(selection) <- selection_ids
    }
    
    # update node selections types to metaviztree
    if(!is.null(selection)) {
      for(n in names(selection)){
        .self$.nodeSelections[[n]] = selection[[n]]
      }
    }
    
    if(!is.null(selectedLevels)) {
      .self$.levelSelected <- as.integer(names(selectedLevels)[1])
    }
    .self$.mgr$.clear_datasourceGroup_cache(.self)
  },
  
  getRows=function(measurements = NULL, start = 1, end = 1000, selectedLevels = 3, selections = NULL) {
    "Return the sample annotation and features within the specified range and level for a given sample and features
    \\describe{
    \\item{measurements}{Samples to retrieve for}
    \\item{start}{Start of feature range to query}
    \\item{end}{End of feature range to query}
    \\item{selections}{Node-id and selectionType pairs}
    \\item{selectedLevels}{Current aggregation level}
    }
    "
    
    nodes_at_level <- .self$.graph@nodes_table[level==selectedLevels, ]
    nodes_at_level_ids <- nodes_at_level[,id]
    
    if(!is.null(selections) && !(length(selections) == 0)){
      nodes_at_level_selections <- rep(2, length(nodes_at_level_ids))
      names(nodes_at_level_selections) <- nodes_at_level_ids
      selections <- c(selections, nodes_at_level_selections)
      
      expand_selections <- which(selections == 1)
      if(!is.null(expand_selections) && length(expand_selections) > 0){
        expand_selection_indices = which(names(selections) %in% names(expand_selections))
        selections <- selections[-expand_selection_indices]
      }
      
      expanded_children <- .self$.graph@nodes_table[parent %in% names(expand_selections),id]
      
      child_lineage <- .self$.graph@nodes_table[id %in% names(selections),]
      remove_selections <- which(selections == 0)
      if(length(remove_selections) > 0){
        kept_nodes <- child_lineage[!grepl(paste(paste(names(remove_selections), collapse=",|"), ",",sep=""), lineage),]
        kept_nodes <- kept_nodes[!(id %in% names(remove_selections)),]
      } else {
        kept_nodes <- child_lineage
      }
      
      agg_selections <- which(selections == 2)
      if(length(agg_selections) > 0){
        kept_nodes <- kept_nodes[!grepl(paste(paste(names(agg_selections), collapse=",|"), ",",sep=""), lineage),]
      }
      kept_nodes <- as.character(kept_nodes[,id])
      nodes_at_level <- .self$.graph@nodes_table[id %in% c(kept_nodes,expanded_children),]
    }
    
    leaf_ordering_table <- as.data.table(.self$.graph@hierarchy_tree[,c(.self$.feature_order[length(.self$.feature_order)], "otu_index")])
    setnames(leaf_ordering_table, c("leaf", "otu_index"))
    leaf_ordering_table <- leaf_ordering_table[,leaf:=as.character(leaf)]
    leaf_ordering_table <- leaf_ordering_table[otu_index >= start & otu_index <= end]
    
    first_join <- merge(leaf_ordering_table, merge(nodes_at_level, .self$.graph@leaf_of_table, by="id"), by="leaf")
    setorderv(first_join, "otu_index.x")
    
    nodes <- as.data.frame(unique(first_join[,c("node_label.x", "lineage.x")]))[,"node_label.x"]
    
    ends <- rep(0, length(nodes))
    starts <- rep(0, length(nodes))
    indexes <- rep(0, length(nodes))
    metadata <- list()
    metadata[['label']] <- list()
    metadata[['id']] <- list()
    metadata[['lineage']] <- list()
    
    for (i in seq_along(nodes)) {
      feature_label <- nodes[i]
      res <- first_join[node_label.x==feature_label,otu_index.x]
      if(length(res) > 0) {
        ends[i] <- max(res)
        starts[i] <- min(res)
        # if(i == length(nodes)) {
        #   starts[i] <- min(res) - 1
        # }
        indexes[i] <- min(res)
        metadata[['label']][i] <- feature_label
        metadata[['id']][i] <- unique(.self$.graph@nodes_table[node_label==nodes[i], id])[1]
        metadata[['lineage']][i] <- unique(.self$.graph@nodes_table[node_label==nodes[i], lineage])[1]
      }
    }
    
    data_rows = list(end=ends, start=starts, index =indexes, metadata=metadata)
    return(data_rows)
  },
  
  searchTaxonomy=function(query = NULL, max_results = 15) {
    "Return list of features matching a text-based query
    \\describe{
    \\item{query}{String of feature for which to search}
    \\item{max_results}{Maximum results to return}
    }
    "
    return(list())
  },
  
  getCombined=function(measurements = NULL,
                       seqName, start = 1, end = 1000,
                       order = NULL, nodeSelection = NULL, selectedLevels = NULL) {
    "Return the counts aggregated to selected nodes for the given samples
    \\describe{
    \\item{measurements}{Samples to get counts for}
    \\item{seqName}{name of datasource}
    \\item{start}{Start of feature range to query}
    \\item{end}{End of feature range to query}
    \\item{order}{Ordering of nodes}
    \\item{nodeSelection}{Node-id and selectionType pairs}
    \\item{selectedLevels}{Current aggregation level}
    }
    "
    
    # update node selections types to tree
    if(!is.null(nodeSelection)) {
      for(n in names(nodeSelection)){
        .self$.nodeSelections[[n]] = nodeSelection[[n]]
      }
    }
    
    if(is.null(selectedLevels)) {
      selectedLevels = .self$.levelSelected + 1
    }
    
    selections = .self$.nodeSelections
    measurements = unique(measurements)
    
    data_rows = .self$getRows(measurements = measurements, start = start, end = end, selectedLevels = selectedLevels, selections = selections)
    row_order = unlist(data_rows$metadata$label)
    aggcounts = aggregateTree(.self$.object, selectedLevel=selectedLevels, selectedNodes=selections, start = start, end = end, by=.self$.treeIn, format="counts")
    
    data_columns = list()
    
    if (.self$.treeIn == "row") {
      for(m in measurements) {
        if(m %in% colnames(aggcounts)){
          inner_result <- aggcounts[,m]
          data_columns[[m]] <- unname(unlist(inner_result))
        } else {
          inner_result <- rep(0.0, nrow(aggcounts))
          data_columns[[m]] <- inner_result
        }
      }
    } else if (.self$.treeIn == "col") {
      for(m in measurements) {
        if(m %in% rownames(aggcounts)){
          inner_result <- aggcounts[m,]
          data_columns[[m]] <- unname(unlist(inner_result))
        } else {
          inner_result <- rep(0.0, ncol(aggcounts))
          data_columns[[m]] <- inner_result
        }
      } 
    }
    
    result <- list(
      cols = data_columns,
      rows = data_rows,
      globalStartIndex = data_rows$start[[1]]
    )
    
    return(result)
  },
  
  getReducedDim=function(method = NULL, gene = NULL) {
    " Compute PCA over all features for given samples
    \\describe{
    \\item{method}{which dimension to access}
    \\item{gene}{send expression of a gene back with the dimensions}
    }
    "
    
    if (is.null(method)) {
      method <- names(metadata(.self$.object)$reduced_dim)[[1]]
    }
    
    data_rows = .self$getRows(measurements = NULL, start = 1, end = 100000,
                              selectedLevels = .self$.levelSelected + 1,
                              selections = .self$.nodeSelections)
    
    max_length <- ncol(.self$.object)

    cluster_names <- rep("removed", max_length)
    for (i in seq_along(data_rows$metadata$label)) {
      start <- data_rows$start[i]
      end <- data_rows$end[i]
      cluster_names[start:end] <- data_rows$metadata$label[i]
    }
    
    measurements <-  metadata(.self$.object)$reduced_dim[[method]]
    
    genes <- rep(100, length(measurements))
    if (!(is.null(gene)) && gene != "") {
      genes <- assays(.self$.object)$counts[gene, ]
    }
    
    data <- list()
    level <- .self$.levelSelected + 1
    i <- 1
    for (col in rownames(measurements)) {
      row_index = which(colData(.self$.object)$samples == col)
      
      # TODO: need to add sample attributes
      temp    <-
        list(
          sample_id = col,
          dim1 = unname(measurements[col, 1]),
          dim2 = unname(measurements[col, 2]),
          name = unlist(cluster_names[[row_index]]),
          gene = unname(genes[row_index])
          # name = name
        )
      data[[col]] <- temp
      i <- i+1
    }
    
    result <- list("data" = unname(data), "pca_variance_explained" = c(1,1),
                   "cluster_order" = data_rows$metadata$label,
                   "gene_min_max" = c(min(genes), max(genes)))
    return(result)
  },
  
  extract_SCE_epiviz = function(cluster_name="treeviz_clusters") {

    data_rows = .self$getRows(measurements = NULL, start = 1, end = 100000,
                              selectedLevels = .self$.levelSelected + 1,
                              selections = .self$.nodeSelections)
    
    max_length <- ncol(.self$.object)

    cluster_names <- rep("removed", max_length)
    for (i in seq_along(data_rows$metadata$label)) {
      start <- data_rows$start[i]
      end <- data_rows$end[i]
      cluster_names[start:end] <- data_rows$metadata$label[i]
    }
    
    # TODO: dumb but whatever, need a shorthand notation
    coldata <- data.frame(treeviz_clusters = unlist(cluster_names))
    colnames(coldata) <- c(cluster_name)
    
    sce <- SingleCellExperiment(assays = list(counts = assays(.self$.object)$counts),
                                colData = coldata,
                                rowData=rowData(.self$.object)
    )
  },
  
  getGeneExpr=function(measurements=NULL){
    
    if (!is.null(measurements)) {
      gene <- measurements[1]
    }
    
    data_rows = .self$getRows(measurements = NULL, start = 1, end = 100000,
                              selectedLevels = .self$.levelSelected + 1,
                              selections = .self$.nodeSelections)
    
    max_length <- ncol(.self$.object)

    cluster_names <- rep("removed", max_length)
    for (i in seq_along(data_rows$metadata$label)) {
      start <- data_rows$start[i]
      end <- data_rows$end[i]
      cluster_names[start:end] <- data_rows$metadata$label[i]
    }
    
    level<- .self$.levelSelected + 1
    selectedNodes <- .self$.nodeSelections
    
    counts <- assays(.self$.object)$counts
    
    gene_col <- counts[gene, ]
    data <- list()
    i <- 1
    for (col in names(gene_col)) {
      
      row_index = which(colData(.self$.object)$samples == col)
      
      data[[col]] <- list(
        alphaDiversity = gene_col[[col]],
        sample_id = col,
        name = unlist(cluster_names[[row_index]])
      )
      i <- i+1
    }
    
    result <- list(data = unname(data))
    return(result)
  }
)

#' TreeViz class wrapper for SummarizedExperiment objects
#' @import SummarizedExperiment
#' @exportClass TreeViz
setClass("TreeViz",
         contains = "SummarizedExperiment")

# validity
setValidity("TreeViz", function(object) {
  msg <- NULL
  if(!(is(rowData(object), "TreeIndex") || is(colData(object), "TreeIndex"))) {
     msg <- "neither rowData nor colData are TreeIndex objects, use SummarizedExperiment instead."
  }

  if(is.null(msg)) TRUE else msg
})

#' The TreeViz class.
#'
#' SummarizedExperiment-like class for datasets that have hierarchies on either rowData or colData.
#' For microbiome data, rowData is a tree hierarchy
#' For single cell data, colData is a tree hierarchy
#' @param assays simple list of counts
#' @param rowData rowData
#' @param colData colData
#' @param ... other parameters for SummarizedExperiment
#' @importFrom S4Vectors SimpleList
#' @importFrom S4Vectors DataFrame
#' @importFrom methods new
#' @examples
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
#' @export
#' @return a `TreeViz` object
TreeViz <- function(assays = SimpleList(),
                   rowData = NULL,
                   colData = NULL,
                   ...) {

  if (missing(colData) || is.null(colData)) {
    assay <- assays[[1]]
    colData <- DataFrame(x=seq_len(ncol(assay)), row.names=colnames(assay))[, FALSE]
  }

  sumExp <- SummarizedExperiment(assays = assays, rowData = rowData, colData = colData, ...)

  new("TreeViz", sumExp)
}

.check_alternate <- function(sub_df, all_df) {
  #check where clusters from not-core edges should actually belong
  assign_df <- data.frame(
    "Sink_node_res" = character(),
    "Sink_node_clust" = character(),
    "Assign_from_res" = character(),
    "Assign_from_clust" = character(),
    "Assign_to_res" = character(),
    "Assign_to_clust" = character(),
    stringsAsFactors = FALSE
  )
  for (i in seq_len(nrow(sub_df))) {
    for (j in seq_len(nrow(all_df))) {
      if (sub_df$to_node[i] == all_df$to_node[j]) {
        assign_df[nrow(assign_df) + 1, ] <-
          c(
            sub_df$to_cluster[i],
            sub_df$to_clust[i],
            sub_df$from_cluster[i],
            sub_df$from_clust[i],
            all_df$from_cluster[j],
            all_df$from_clust[j]
          )
      }
    }
  }
  
  assign_df
}


.change_assignment <- function(graph, cluster_obj) {
  #for each row in the core edge false dataset, find out corresponding
  #source and sink in Seurat, and change accordingly
  for (i in seq_len(nrow(graph))) {
    sink_res = match(paste0('cluster', graph$`Sink_node_res`[i]),
                     colnames(cluster_obj))
    source_res = match(paste0('cluster', graph$`Assign_from_res`[i]),
                       colnames(cluster_obj))
    
    
    for (j in seq_len(nrow(cluster_obj))) {
      
      if (as.numeric(as.character(cluster_obj[[j, sink_res]])) == as.numeric(graph$Sink_node_clust[i])  &&
          as.numeric(as.character(cluster_obj[[j, source_res]])) == as.numeric(graph$Assign_from_clust[i])) {
        cluster_obj[[j, source_res]] <-
          as.factor(graph$`Assign_to_clust`[i])
      }
      
    }
  }
  
  return (cluster_obj)
}

#' @importFrom utils str
#' @import igraph
.check_cycle <- function(pruned_graph) {
  # Check if a diamond or circle still exists in tree
  delete_set_vertices <- vector('numeric')
  ver_list <- V(pruned_graph)
  
  for (nodes in ver_list) {
    adj_edge <- incident(pruned_graph, nodes, mode = "in")
    
    if (length(adj_edge) > 1) {
      # Not a Tree yet
      adj_ver <-
        adjacent_vertices(pruned_graph, nodes, mode = "in")
      adj_ver <- as_ids(adj_ver[[1]])
      str(adj_ver)
      remove_node <- sample(adj_ver, 1)
      
      remove_edge <-
        incident(pruned_graph, remove_node, mode = "all")
      
      pruned_graph <- delete_edges(pruned_graph, remove_edge)
      delete_set_vertices <-
        c(delete_set_vertices, remove_node)
      
    }
  }
  pruned_graph <- delete.vertices(pruned_graph, delete_set_vertices)
}

.prune_tree <- function(graph, cluster_df) {
  # Prune the tree so only core edges remain
  
  repeat {
    # drop duplicated columns (Throws error otherwise)
    
    graph <- graph[!duplicated(names(graph))]
    
    # See number of core edges at each iter
    # No False edges acyclic tree
    if (nrow(graph[graph$is_core == FALSE, ]) == 0) {
      ngraph <-
        clustree(
          cluster_df ,
          prefix = "cluster",
          prop_filter = 0,
          return = "graph"
        )
      break
    }
    
    # Apply function
    assign_df <-
      .check_alternate(graph[graph$is_core == FALSE, ], graph[graph$is_core == TRUE, ])
    
    cluster_df <- .change_assignment(assign_df, cluster_df)
    ngraph <-
      clustree(
        cluster_df ,
        prefix = "cluster",
        prop_filter = 0,
        return = "graph"
      )
    
    graph <- as_long_data_frame(ngraph)
  }
  
  ngraph <- .check_cycle(ngraph)
  
  result <-
    list("Cluster_obj" = cluster_df, "Clustree_obj" = ngraph)
  result
}

.collapse_tree <- function(original_graph) {
  #find all roots
  root_list <- which(
    vapply(
      V(original_graph),
      function(x) {
        length(neighbors(original_graph, x, mode = "in")) == 0
      },
      logical(1)
    ))
  
  delete_set_vertices <- vector('numeric')
  for (roots in root_list) {
    node_dists <- distances(original_graph, to = roots)
    layered_dist <- unique(distances(original_graph, to = roots))
    
    layered_dist <- layered_dist[is.finite(layered_dist) == TRUE]
    # Vertex and edge lists of the graph from which we will construct the collapsed graph
    
    ver_list <-
      igraph::as_data_frame(original_graph, what = "vertices")
    
    
    i <- length(layered_dist)
    while (i >= 2) {
      prev_layer <- which(node_dists == layered_dist[i - 1])
      current_layer <- which(node_dists == layered_dist[i])
      
      if (length(prev_layer) == length(current_layer)) {
        while (length(prev_layer) == length(current_layer)) {
          delete_set_vertices <- c(delete_set_vertices, prev_layer)
          i <- i - 1
          
          prev_layer <-
            which(node_dists == layered_dist[i - 1])
          
        }
      }
      i <- i - 1
    }
  }
  
  if (length(delete_set_vertices) != 0) {
    message("Collapsing the graph by removing redundant nodes")
    ver_list <- ver_list[-delete_set_vertices, ]
  }
  ver_list
}

.checkRoot <- function(cluster_df) {
  # Handle Forests
  cols <- colnames(cluster_df)
  if (length(unique(cluster_df[[1]])) > 1) {
    message("Given Tree is a Forest\n Adding extra root")
    cluster_df$root <- "ClusterAllClusters"
    cols <- c("root", cols)
  }
  
  cluster_df[cols]
}

.check_unique_parent <- function(clusterdata) {
  #check if user provided data has unique parents at each level
  for (i in seq(2, ncol(clusterdata))) {
    childs <- unique(clusterdata[[i]])
    for (values in childs) {
      subsetted_list <-
        clusterdata[clusterdata[[colnames(clusterdata)[[i]]]] == values, ]
      
      
      parent <- length(unique(subsetted_list[[i - 1]]))
      
      if (parent > 1) {
        message(
          "Not a tree, some nodes with multiple parents in level ",
          i,
          "\n Performing Cluster reassignment "
        )
        return (FALSE)
      }
    }
  }
  return(TRUE)
}


.rename_clusters <- function(clusterdata) {
  message("Renaming cluster Levels...")
  message("Previous Level names ", paste(colnames(clusterdata), collapse = "\t"))
  
  clusnames <- seq_len(length(colnames(clusterdata)))
  clusnames <- paste0("cluster", clusnames)
  colnames(clusterdata) <- clusnames
  message("New Level names ", paste(colnames(clusterdata), collapse = "\t"))
  clusterdata
}


#' @import clustree
#' @importFrom data.table setorder
#' @importFrom igraph as_long_data_frame
.preprocessAndCreateTreeViz <- function(clusters, counts) {
  clusters <- .rename_clusters(clusters)
  setorder(clusters)
  counts <- counts[, rownames(clusters)]
  
  # Create clustree object
  clustree_graph <-
    clustree(
      clusters,
      prefix = "cluster",
      prop_filter = 0,
      return = "graph"
    )
  
  graph_df <- as_long_data_frame(clustree_graph)
  
  # prune the graph with only core edges (this makes it a ~tree)
  modified_obj <- .prune_tree(graph_df, clusters)
  
  # modified graph and seurat object
  modified_graph <- modified_obj$Clustree_obj
  clusters_new <-  modified_obj$Cluster_obj
  
  # collapses tree if the levels are the same at different resolutions
  collapsed_graph <- .collapse_tree(modified_graph)
  cluster_names <- unique(vapply(strsplit(collapsed_graph$node, "C"), '[', chracter(1), 1))
  clusters_new <- clusters_new[, cluster_names]
  
  for (clusnames in names(clusters_new)) {
    clusters_new[[clusnames]] <-
      paste(clusnames, clusters_new[[clusnames]], sep = 'C')
  }
  
  samples <- rownames(clusters_new)
  clusters_new <- cbind(clusters_new, samples)
  clusters_new <- .checkRoot(clusters_new)
  
  tree <- TreeIndex(clusters_new)
  rownames(tree) <- rownames(clusters_new)
  
  treeviz <- TreeViz(SimpleList(counts = counts), colData = tree)
  treeviz
}

.generate_walktrap_hierarchy <- function(object, nsteps = 7) {
  
  message("calculating walktrap clusters")
  SNN_Graph <- scran::buildSNNGraph(object)
  clusters <- igraph::cluster_walktrap(SNN_Graph, steps = nsteps)
  modularity <- c()
  
  for (i in seq_len(length(clusters))) {
    modularity[i] <-
      igraph::modularity(SNN_Graph, igraph::cut_at(clusters, n = i))
  }
  
  monotonic_index <- match(unique(cummax(modularity)), modularity)
  cluster_data =  list()
  for (i in seq_along(monotonic_index)) {
    cluster_data[[i]] =  list(igraph::cut_at(clusters, n = monotonic_index[i]))
  }
  
  cluster_data <- as.data.frame(cluster_data)
  colnames(cluster_data) <- paste0("cluster", monotonic_index)
  cluster_data$samples <- rownames(cluster_data) <- colnames(object)
  cluster_data
}

#' Creates a `TreeViz` object from `Seurat`
#'
#' @param object `Seurat` class containing cluster information at different resolutions
#' @param check_metadata whether to metaData of `Seurat` object for cluster information or not
#' @param col_regex common regular expression shared across all columns with cluster information
#' @param columns vector containing columns with cluster information
#' @param reduced_dim Vector of Dimensionality reduction information provided in `Seurat` object to be added in `TreeViz` (if exists)
#' @return `TreeViz` Object
#' @examples
#' library(Seurat)
#' data(pbmc_small)
#' pbmc <- pbmc_small
#' treeviz<- createFromSeurat(pbmc, check_metadata = TRUE, reduced_dim = c("pca","tsne"))
#' @importFrom Seurat as.SingleCellExperiment
#' @importFrom Seurat GetAssayData
#' @importFrom Seurat Reductions
#' @export
createFromSeurat <- function(object,
                             check_metadata = FALSE,
                             col_regex = "*snn*",
                             columns = NULL,
                             reduced_dim = c("TSNE")) {
  
  if (check_metadata==FALSE) {
    message("No default clusters provided")
    object.sce <- as.SingleCellExperiment(object)
    clusterdata <- .generate_walktrap_hierarchy(object.sce)
    clusterdata <- ClusterHierarchy(clusterdata)
  }
  else{
    clusterdata <- object@meta.data
    clusterdata$samples <- rownames(clusterdata) <- colnames(object)
    clusterdata <- ClusterHierarchy(clusterdata, col_regex, columns)
  }
  
  treeviz <- createTreeViz(clusterdata, GetAssayData(object))
  
  for (dim_names in reduced_dim) {
    if (dim_names %in% Reductions(object)) {
      reducdim <- Reductions(object, slot = dim_names)
      
      metadata(treeviz)$reduced_dim[[dim_names]] <- reducdim@cell.embeddings[, c(1,2)]
      rownames(metadata(treeviz)$reduced_dim[[dim_names]]) <- colnames(object)
    }
  }
  
  print(names(metadata(treeviz)$reduced_dim))
  treeviz
}

#' Creates a `TreeViz`` object from `SingleCellExperiment`. Generates
#' clusters based on Walktrap algorithm if no default is provided
#' @param object `SingleCellExperiment` object to be visualized
#' @param check_coldata whether to colData of `SingeCellExperiment` object for cluster information or not
#' @param col_regex common regular expression shared across all columns with cluster information
#' @param columns vector containing columns with cluster information
#' @param reduced_dim Vector of Dimensionality reduction information provided in `SingeCellExperiment` object to be added in `TreeViz` (if exists)
#' @return `TreeViz` Object
#' @examples
#' library(SingleCellExperiment)
#' library(scater)
#' sce <- mockSCE()
#' sce <- logNormCounts(sce)
#' sce <- runTSNE(sce)
#' sce <- runUMAP(sce)
#' set.seed(1000)
#' for (i in  seq_len(5)) {
#' clust.kmeans <- kmeans(reducedDim(sce, "TSNE"), centers = i)
#' sce[[paste0("clust", i)]] <- factor(clust.kmeans$cluster)
#' }
#' treeviz <-createFromSCE(sce, check_coldata = TRUE, col_regex = "clust", reduced_dim = c("TSNE", "UMAP"))
#' 
#' @import SingleCellExperiment
#' @export
createFromSCE <-
  function(object,
           check_coldata = FALSE,
           col_regex = NULL,
           columns = NULL,
           reduced_dim = c("TSNE")) {
    
    if (check_coldata == TRUE) {
      clusterdata <- colData(object)
      clusterdata$samples <- rownames(clusterdata) <- colnames(object)
      clusterdata <- ClusterHierarchy(clusterdata, col_regex, columns)
    }
    else{
      message("No default clusters provided")
      clusterdata <- .generate_walktrap_hierarchy(object)
      clusterdata$samples <- rownames(clusterdata) <- colnames(object)
      clusterdata <- ClusterHierarchy(clusterdata)
    }
    
    count <- counts(object)
    rownames(count) <- rownames(counts(object))
    
    treeviz <- createTreeViz(clusterdata, count)
    for (dim_names in reduced_dim) {
      if (dim_names %in% reducedDimNames(object)) {
        metadata(treeviz)$reduced_dim[[dim_names]] <-
          reducedDims(object)[[dim_names]][, c(1,2)]
        
        rownames(metadata(treeviz)$reduced_dim[[dim_names]]) <- colnames(object)
      }
    }
    
    treeviz
  }

#' Creates `TreeViz` object from hierarchy and count matrix
#' 
#' Provided with a count matrix and a dataframe or `ClusterHierarchy` object, this module 
#' runs the necessary checks on the dataframe and tries to convert it to a tree by making necessary changes.
#' Returns the `TreeViz` object if a tree is successfully generated from dataframe, throws error otherwise
#' @param clusters `ClusterHierarchy` object or a dataframe containing cluster information at different resolutions
#' @param counts matrix Dense or sparse matrix containing the count matrix
#' @return `TreeViz`` Object
#' @examples
#' n=64
#' # create a hierarchy
#' df<- data.frame(cluster0=rep(1,n))
#' for(i in seq_len(5)){
#'   df[[paste0("cluster",i)]]<- rep(seq(1:(2**i)),each=ceiling(n/(2**i)),len=n)
#' }
#' # generate a count matrix
#' counts <- matrix(rpois(6400, lambda = 10), ncol=n, nrow=100)
#' colnames(counts)<- seq_len(64)
#' # create a `TreeViz` object
#' treeViz <- createTreeViz(df, counts)
#' 
#' @export
createTreeViz <- function(clusters, counts) {
  
  if (is(clusters,"data.frame") ){
    clusters$samples <- rownames(clusters) <- colnames(counts)
    clusters <- ClusterHierarchy(clusters)
  }
  
  
  if (!is(clusters, "ClusterHierarchy")) {
    stop("clusters is not a ClusterHierarchy class")
  }
  
  counts <- counts[,clusters$samples]
  column_names <- colnames(clusters)
  clusters <- as.data.frame(clusters)
  colnames(clusters) <- column_names
  
  tree <- TreeIndex(clusters)
  rownames(tree) <- rownames(clusters)
  treeviz <- TreeViz(SimpleList(counts = counts), colData = tree)
  treeviz
}


#' @importFrom scran getTopHVGs
#' @importFrom scran modelGeneVar
.find_top_variable_genes <- function(treeviz, top = 100) {
  dec.treeviz <- modelGeneVar(assays(treeviz)$counts)
  top_n <- getTopHVGs(dec.treeviz, n = top)
  metadata(treeviz)$top_variable <- top_n
  
  treeviz
}

#' Sets gene list for visualization
#' @param treeviz TreeViz object
#' @param genes list of genes to use
#' @return TreeViz object set with gene list
set_gene_list <- function(treeviz, genes) {
  genes_in_assay  <- rownames(assays(treeviz)$counts)
  metadata(treeviz)$top_variable <- intersect(genes_in_assay, genes)
  
  treeviz
}

#' @importFrom scater calculateTSNE
.calculate_tsne <- function(treeviz) {
  message("No defaults dimensionality reductions provided")
  message("Calculating TSNE")
  tsne <- calculateTSNE(assays(treeviz)$counts)
  metadata(treeviz)$reduced_dim[['TSNE']] <- tsne[, c(1,2)]
  rownames(metadata(treeviz)$reduced_dim[['TSNE']]) <- colnames(treeviz)
  message("adding tsne to reduced dim slots")
  treeviz
}

context("testing TreeViz Class")
library(Seurat)
library(scater)

n=32
df <- data.frame(cluster0=rep(seq_len(2),each=ceiling(n/(2)),len=n))

for(i in seq_len(2)){
  df[[paste0("cluster",2*i-1)]]<- rep(seq_len((2**i)),each=ceiling(n/(2**i)),len=n)
  df[[paste0("cluster",2*i)]]<- rep(seq_len((2**i)),each=ceiling(n/(2**i)),len=n)
}

df[[paste0("cluster",2*i+1)]]<- rep(seq_len(7),each=ceiling(n/7),len=n)
counts <- matrix(rpois(3200, lambda = 10), ncol=n, nrow=100)

for(cols in colnames(df)){
  df[[cols]] <- as.factor(df[[cols]])
}

colnames(counts) <- df$samples <- rownames(df) <- paste0("cell-", 1:ncol(counts))
rownames(counts) <- paste0("gene-", 1:nrow(counts))
hierarchy <- ClusterHierarchy(df)

treeviz <- createTreeViz(hierarchy, counts)

test_that("create TreeVizClass", {
  expect_is(treeviz, "TreeViz")

})

test_that("check tree_collapse", {
  expect_equal(6, ncol(colData(treeviz)@hierarchy_tree))
})

test_that("check single_root", {
  expect_equal("ClusterAllClusters", unique(colData(treeviz)@hierarchy_tree[[1]]))
})

test_that("check multiple_parent", {

  hierarchydf <- colData(treeviz)@hierarchy_tree
  hierarchydf <- hierarchydf[, !colnames(hierarchydf) %in% c("samples", "otu_index")]
  
  flag <- TRUE
  
  childs <- unique(hierarchydf[[ncol(hierarchydf)]])
  for (values in childs) {
    subsetted_list <- hierarchydf[hierarchydf[[ncol(hierarchydf)]] == values,]
    parent <- length(unique(subsetted_list[[ncol(hierarchydf) - 1]]))
    if (parent > 1) {
      flag <- FALSE
    }
  }
  
  expect_equal(flag, TRUE)
})

test_that("check aggregate_tree", {
  sums <- list()
  sums[[1]] <- list(rowSums(counts[,c(1:10)]))
  sums[[2]] <- list(rowSums(counts[,c(11:15)]))
  sums[[3]] <- list(rowSums(counts[,c(16:25)]))
  sums[[4]] <- list(rowSums(counts[,c((26:32))]))
  sum_df <- as.data.frame(sums)
  aggtree <- scTreeViz:::aggregateTree(treeviz, by="col", selectedLevel=3)
  
  agg_df <- as.data.frame(assays(aggtree)$counts)
  # result <- all.equal(agg_df, sum_df, check.attributes= FALSE)

  expect_equal(dim(agg_df),dim(sum_df))
  result<- all.equal(agg_df, sum_df, check.attributes= FALSE, ignore.col.order=TRUE, ignore.row.order=TRUE)

  expect_equal(result, TRUE)
})

test_that("Testing CreatefromSeurat class", {
  skip("conflict between new Seurat and Summarized Experiment")
  pbmc_small
  treeviz<-createFromSeurat(pbmc_small)
  expect_is(treeviz, "TreeViz")
})

test_that("Testing CreatefromSCE class", {
  #skip("need data here")
  set.seed(1)
  sce <- mockSCE()
  sce <- logNormCounts(sce)
  treeviz<-createFromSCE(sce) 
  expect_is(treeviz, "TreeViz")
})

#' show object
#' @param object TreeViz object
#' @importFrom methods show
#' @importFrom S4Vectors metadata
#' @export
#' 
#' @describeIn TreeViz-methods
#' 
#' @return describe a TreeIndex object
setMethod("show", signature("TreeViz"),
          function(object) {
            cat("class: TreeViz \n", sep = " ")
            cat("dim:", nrow(object), ncol(object), "\n", sep = " ")
            cat("metadata:\n")
            cat(show(names(metadata(object))))
            cat("rowData:\n")
            cat(show(rowData(object)), "\n")
            cat("colData:\n")
            cat(show(colnames(colData(object))))
          })

#' Method to aggregate a TreeViz object
#' @param x object
#' @param ... other params to the function
#' 
#' @describeIn TreeViz-methods
#' 
#' @examples
#' \donttest{ 
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
#' aggregateTree(mbiome)
#' }
#' @return a generic
#' @export
setGeneric("aggregateTree", signature = "x",
           function(x, ...)
             standardGeneric("aggregateTree"))

#' Method to aggregate a TreeViz object
#' @param x TreeViz object
#' @param selectedLevel level to select nodes from
#' @param selectedNodes used to set states on individual nodes to define a cut on the tree
#' @param start,end indices to filter nodes
#' @param by "row" to aggregate the TreeIndex on rowData, "col" to aggregate TreeIndex on colData
#' @param aggFun aggregate function to use, by default colSums if by="row", rowSums if by="col"
#' @param format return format can be one of "counts" or "TreeViz"
#' @importFrom Matrix rowSums colSums
#' @return a Treeviz object or type specified by format
#' 
#' @describeIn TreeViz-methods
#' 
#' @examples
#' \donttest{ 
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
#' aggregateTree(mbiome)
#' }
#' @export
setMethod("aggregateTree", "TreeViz",
          function(x,
                   selectedLevel = 3,
                   selectedNodes = NULL,
                   aggFun = colSums,
                   start = 1,
                   end = NULL,
                   by = "row",
                   format = "TreeViz") {
            if (is.null(end) || missing(end)) {
              end <- nrow(x)
            }
            
            if(is(selectedNodes, "data.table")) {
              node_ids <- selectedNodes$id
              snodes <- rep(1, length(node_ids))
              
              if("state" %in% colnames(selectedNodes)) {
                snodes <- selectedNodes$state
              }
              names(snodes) <- node_ids
              selectedNodes <- snodes
            }
            
            if (by == "row") {
              aggFun <- colSums
              groups <-
                splitAt(
                  rowData(x),
                  selectedLevel = selectedLevel,
                  selectedNodes = selectedNodes,
                  start = start,
                  end = end,
                  format = "list"
                )
              counts <-  assays(x)$counts
              
              newMat <- array(NA, dim = c(length(groups), ncol(x)))
              for (i in seq_along(groups)) {
                indices <- groups[[i]]
                if (length(indices) == 1) {
                  newMat[i, ] = counts[indices, ]
                }
                else {
                  newMat[i, ] = aggFun(counts[indices, ])
                }
              }
              
              rownames(newMat) <- names(groups)
              colnames(newMat) <- colnames(x)
              
            }
            else if (by == "col") {
              aggFun <- rowSums
              groups <-
                splitAt(
                  colData(x),
                  selectedLevel = selectedLevel,
                  selectedNodes = selectedNodes,
                  start = start,
                  end = end,
                  format = "list"
                )
              counts <-  assays(x)$counts
              
              newMat <- array(NA, dim = c(nrow(x), length(groups)))
              for (i in seq_along(groups)) {
                indices <- groups[[i]]
                if (length(indices) == 1) {
                  newMat[, i] = counts[, indices]
                }
                else {
                  newMat[, i] = aggFun(counts[, indices])
                }
              }
              
              colnames(newMat) <- names(groups)
              rownames(newMat) <- rownames(x)
            }
            
            if(!is.null(selectedNodes)) {
              return(newMat)
            }
            
            if (format == "TreeViz") {
              if (by == "row") {
                newRowData <-
                  splitAt(
                    rowData(x),
                    selectedLevel = selectedLevel,
                    selectedNodes = selectedNodes,
                    start = start,
                    end = end,
                    format = "TreeIndex"
                  )
                
                newColData <- colData(x)
              }
              else if (by == "col") {
                newRowData <- rowData(x)
                newColData <-
                  splitAt(
                    colData(x),
                    selectedLevel = selectedLevel,
                    selectedNodes = selectedNodes,
                    start = start,
                    end = end,
                    format = "TreeIndex"
                  )
              }
              
              newSumExp <-
                SummarizedExperiment(SimpleList(counts = newMat), rowData = newRowData, colData = newColData)
              
              newTreeSE <- new("TreeViz", newSumExp)
              
              return(newTreeSE)
            }
            else if (format == "counts") {
              return(newMat)
            }
          })

#' Generic method to register data to the epiviz data server
#'
#' @param object The object to register to data server
#' @param tree Is tree over rows or columns of the object (default: "row")
#' @param columns Name of columns containing data to register
#' @param ... Additional arguments passed to object constructors
#' @return An \code{\link{EpivizTreeData-class}} object
#' 
#' @describeIn TreeViz-methods
#' 
#' @importMethodsFrom epivizrData register
#'
setMethod("register", "TreeViz", function(object, tree="row", columns=NULL, ...) {
  return(EpivizTreeData$new(object=object, tree=tree, columns=columns, ...))
})

#' plot tree from TreeViz
#'
#' @param x treeviz object
#' @param y none
#' @return Dataframe containing cluster information at different resolutions
#' 
#' @import ggraph
#' @importFrom igraph graph_from_data_frame
#' @importFrom ggplot2 aes
#' 
#' @describeIn TreeViz-methods
#' 
#' @examples
#' \donttest{
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
#' plot(mbiome)
#' }
#' @export
#' 
setMethod("plot", "TreeViz", function(x, y) {
  object <- x
  
  if(is(colData(object), "TreeIndex")) {
    hierarchydf <- colData(object)@hierarchy_tree
  }
  else {
    hierarchydf <- rowData(object)@hierarchy_tree
  }
  
  
  hierarchydf <- hierarchydf[, !colnames(hierarchydf) %in% c("samples", "otu_index")]
  
  df <- data.frame(from = numeric(), to = numeric())
  
  for (i in seq_len(ncol(hierarchydf) - 1)) {
    edges<- hierarchydf[,c(i,i+1)]
    edges<-unique(edges)
    colnames(edges)<- c("from","to")
    df <- rbind(df, edges)
  }
  
  mygraph <- graph_from_data_frame(df)
  .nms <- V(mygraph)$name
  .nm_suffix <- vapply(.nms, function(nm) substring(nm, first=length(nm)-4, last=length(nm)), character(1))
  V(mygraph)$plot_label <- 
    paste0(substring(.nms, first=1, last=4), "...", .nm_suffix)


  fig <- ggraph(mygraph, layout = 'dendrogram', circular = FALSE) +
    ggraph::geom_edge_diagonal() +
    ggraph::geom_node_point(show.legend = TRUE) +
    ggplot2::theme_void()
  show(fig)
  
})

.register_all_treeviz_things <- function(app) {
  
  app$server$register_action("registerChartTypes", function(request_data) {
    app$chart_mgr$.register_available_chart_types(request_data$data)
  })
  
  app$server$register_action("getHierarchy", function(request_data) {
    datasource = request_data$datasourceGroup
    nodeId = request_data$nodeId
    obj <- app$data_mgr$.find_datasource(datasource)
    if (is.null(obj)) {
      stop("cannot find datasource", datasource)
    }
    obj$getHierarchy(nodeId)
  })
  
  app$server$register_action("propagateHierarchyChanges", function(request_data) {
    
    datasource = request_data$datasourceGroup
    selection = request_data$selection
    order = request_data$order
    selectedLevels <- request_data$selectedLevels
    
    obj <- app$data_mgr$.find_datasource(datasource)
    if (is.null(obj)) {
      stop("cannot find datasource", datasource)
    }
    
    obj$propagateHierarchyChanges(selection, order, selectedLevels)
  })
  
  app$server$register_action("getRows", function(request_data) {
    datasource <- request_data$datasource
    seqName <- request_data$seqName
    start <- request_data$start
    end <- request_data$end
    metadata <- request_data$metadata
    
    obj <- app$data_mgr$.find_datasource(datasource)
    if (is.null(obj)) {
      stop("cannot find datasource", datasource)
    }
    obj$getRows(seqName, start, end, metadata)
  })
  
  app$server$register_action("getValues", function(request_data) {
    datasource <- request_data$datasource
    measurement <- request_data$measurement
    seqName <- request_data$seqName
    start <- request_data$start
    end <- request_data$end
    
    obj <- app$data_mgr$.find_datasource(datasource)
    if (is.null(obj)) {
      stop("cannot find datasource", datasource)
    }
    obj$getValues(measurement, seqName, start, end)
  })
  
  app$server$register_action("getCombined", function(request_data) {
    
    measurementsList <- request_data$measurements
    result <- lapply(names(measurementsList), function(m) {
      seqName <- request_data$seqName
      start <- request_data$start
      end <- request_data$end
      order <- request_data$order
      nodeSelection <- request_data$selection
      selectedLevels <- request_data$selectedLevels
      measurements <- measurementsList[[m]]
      
      if(is.null(start)) {
        start <- 0
      }
      else {
        start <- start
      }
      
      if(is.null(end)) {
        end <- 100000
      }
      else {
        end <- end
      }
      
      obj <- app$data_mgr$.find_datasource(m)
      if (is.null(obj)) {
        stop("cannot find datasource", m)
      }
      res <- obj$getCombined(measurements, seqName, start, end, order, nodeSelection, selectedLevels)
      res
    })
    names(result) <- names(measurementsList)
    result
  })
  
  app$server$register_action("splinesSettings", function(request_data) {
    updateAlpha <- as.double(request_data$settings$alpha)
    names_list <- ls(app$data_mgr$.ms_list)
    class_list <- lapply(names_list, function(i) {class(app$data_mgr$.get_ms_object(i))})
    obj <- app$data_mgr$.get_ms_object(names_list[which(class_list == "EpivizMetagenomicsDataTimeSeries")][1])
    obj$updateSplineAlpha(updateAlpha)
    return(list())
  })
  
  app$server$register_action("getSeqInfos", function(request_data) {
    return(list(
      "treevizr" = c(0, 100000)
    ))
  })
  
  app$server$register_action("partitions", function(request_data) {
    return(list(
      "treevizr" = c(0, 100000)
    ))
  })
  
  app$server$register_action("getPCA", function(request_data) {
    
    obj <- app$data_mgr$.find_datasource(request_data$datasource)
    if (is.null(obj)) {
      stop("cannot find datasource", request_data$measurements)
    }
    
    result <- obj$getReducedDim(method=request_data$measurements[[request_data$datasource]][1],
                                gene=request_data$gene)
    result <- list(data = result)
    names(result) <- request_data$datasource
    result
    
  })
  
  app$server$register_action("getDiversity", function(request_data) {
    
    measurementsList <- request_data$measurements
    result <- lapply(names(measurementsList), function(m) {
      seqName <- request_data$seqName
      measurements <- measurementsList[[m]]
      
      obj <- app$data_mgr$.find_datasource(m)
      if (is.null(obj)) {
        stop("cannot find datasource", m)
      }
      obj$getGeneExpr(measurements)
    })
    names(result) <- names(measurementsList)
    result
    
  })
  
  app$server$register_action("search", function(request_data) {
    query <- request_data$q
    max_results <- request_data$maxResults
    datasource = request_data$datasourceGroup
    
    obj <- app$data_mgr$.find_datasource(datasource)
    if (is.null(obj)) {
      stop("cannot find datasource", datasource)
    }
    
    list(nodes = obj$searchTaxonomy(query, max_results))
  })
  
  app$server$register_action("setChartSettings", function(request_data) {
    app$chart_mgr$.update_chart_settings(request_data)
  })
}

.wait_until_connected <- function(server, timeout=60L) {
  ptm <- proc.time()
  while (!server$is_socket_connected() && (proc.time() - ptm < timeout)["elapsed"]) {
    Sys.sleep(0.001)
    server$service()
  }
  if (!server$is_socket_connected()) {
    stop("[epivizrStandalone] Error starting app. UI unable to connect to websocket server.")
  }
  invisible()
}

.delay_requests <- function(server, timeout=2L) {
  ptm <- proc.time()
  
  while ((proc.time() - ptm < timeout)["elapsed"]) {
    Sys.sleep(0.001)
    server$service()
  }
  invisible()
}

.viewer_option_browse_fun <- function(url) {
  viewer <- getOption("viewer")
  if (is.null(viewer)) {
    utils::browseURL(url)
  } else {
    viewer(url)
  }
}


#' Start treeviz app and create \code{\link[scTreeViz]{TreeVizApp}} object to manage connection.
#'
#' @param data TreeViz object to explore
#' @param genes (character vector) genes (rownames) to include in heatmap
#' @param top_genes (integer) number of top variable genes to include in the heatmap 
#' @param host (character) host address to launch.
#' @param register_function (function) function used to register actions and charts on the treeviz app.
#' @param delay (integer) number of seconds to wait for application to load in browser
#' @param ... additional parameters passed to \code{\link[epivizr]{startEpiviz}}.
#'
#' @return An object of class \code{\link[scTreeViz]{TreeVizApp}}
#'
#' @import epivizr
#' @import sys
#'
#' @seealso \code{\link[scTreeViz]{TreeVizApp}}
#' @examples
#' # see package vignette for example usage
#' app <- startTreeviz(non_interactive=TRUE, open_browser=FALSE)
#' app$stop_app()
#'
#' @export
startTreeviz <- function(data = NULL, genes=NULL, top_genes=100, host="http://epiviz.cbcb.umd.edu/treeviz",
                         register_function = .register_all_treeviz_things, delay=2L,
                         ...) {
  chr="treevizr"
  start <- 0
  end <- 100000
  app <- startEpiviz(host = host, register_function = register_function,
                     chr=chr, start=start, end=end, ...)
  mApp <- TreeVizApp$new(.url_parms=app$.url_parms, .browser_fun=app$.browser_fun,
                         server=app$server, data_mgr=app$data_mgr, chart_mgr=app$chart_mgr)
  
  tryCatch({
    
    send_request <- mApp$server$is_interactive()
    
    if (mApp$server$is_interactive()) {
      .wait_until_connected(mApp$server)
    }
    
    if (!is.null(data)) {
      
      if (!is.null(genes)) {
        data <- set_gene_list(data, genes)
      } else {
        data <- .find_top_variable_genes(data, top_genes)
      }

      if (!("reduced_dim"  %in% names(metadata(data)))) {
        data <- .calculate_tsne(data)
      }
      
      mApp$navigate(chr, start, end)
      mApp$server$wait_to_clear_requests()
      .delay_requests(mApp$server, timeout =  delay)
      
      # facetZoom
      facetZoom <- mApp$data_mgr$add_measurements(data, datasource_name = "SCRNA", tree = "col", datasource_origin_name="scrna", send_request=send_request)
      mApp$server$wait_to_clear_requests()
      .delay_requests(mApp$server, timeout = delay)
      
      ms_list <- facetZoom$get_measurements()
      mea <- ms_list[[1]]
      mea@id <- "all"
      
      mApp$chart_mgr$visualize(chart_type = "epiviz.ui.charts.tree.Icicle",  measurements = list(mea))
      mApp$server$wait_to_clear_requests()
      .delay_requests(mApp$server, timeout = delay)
      
      mApp$chart_mgr$visualize(chart_type = "HeatmapPlot",  measurements = list(mea))
      mApp$server$wait_to_clear_requests()
      .delay_requests(mApp$server, timeout=delay)
      
      if ("reduced_dim" %in% names(metadata(data))) {
        for (dim in names(metadata(data)$reduced_dim)) {
          ms_list <- facetZoom$get_measurements()
          mea <- ms_list[[1]]
          mea@id <- dim
          
          # TSNE
          mApp$chart_mgr$visualize(chart_type = "TSNEPlot", measurements = list(mea))
          mApp$server$wait_to_clear_requests()
          .delay_requests(mApp$server, timeout=delay)
        }        
      }
    }
  }, error=function(e) {
    mApp$stop_app()
    stop(e)
  })
  
  mApp
}


.viewer_option_browse_fun <- function(url) {
  viewer <- getOption("viewer")
  if (is.null(viewer)) {
    utils::browseURL(url)
  } else {
    viewer(url)
  }
}

context("create TreeVizApp class")
library(Seurat)
library(scater)

test_that("startTreeViz creates a TreeVizApp Object from Seurat", {
  skip("need data here")
  app <- startTreeviz(non_interactive=TRUE)
  expect_is(app, "TreeVizApp")
  
  expect_is(app$server, "EpivizServer")
  expect_is(app$chart_mgr, "EpivizChartMgr")
  expect_is(app$data_mgr, "EpivizDataMgr")
  
  expect_true(app$server$is_closed())
})

#' ClusterHierarchy class to manage treeviz cluster data
setClass(
  "ClusterHierarchy",
  contains = c("DFrame")
)


#' Creates a new ClusterHierarchy object. 
#' 
#' Works as a validation check for multiple issues user passed dataframe might have.
#' For example, multiple root nodes, incompatible naming, multiple parents of a single node, etc
#' This function performs all this checks and tries to resolve the issues by making changes in cluster assignment
#' User can give either
#' col_regex or  columns option to filter the columns or specify 
#' the column order
#' @param hierarchy hierarchy as a dataFrame
#' @param col_regex Regular Expression for choosing columns
#' @param columns Vector containing list of columns to choose from with ordering
#' @return `ClusterHierarchy`` return an object of class ClusterHierarchy containing cluster information that ensures a valid dataframe for treeviz input
#' @importFrom methods new
#' @importFrom S4Vectors DataFrame
#' @export
#' @examples
#' n=64
#' # create a hierarchy
#' df<- data.frame(cluster0=rep(1,n))
#' for(i in seq(1,5)){
#'   df[[paste0("cluster",i)]]<- rep(seq(1:(2**i)),each=ceiling(n/(2**i)),len=n)
#' }
#' clus_hier<-ClusterHierarchy(df, col_regex = "clus")
#' 
ClusterHierarchy <- function(hierarchy, col_regex=NULL, columns =NULL) {

  #Filter key words
  if(!is.null(col_regex) && !is.null(columns)){
    message("Cannot use both")
  }
  if(!is.null(col_regex)){
      hierarchy <- hierarchy[, grep( col_regex, colnames(hierarchy))] 
  }
  
  # ordering
  if(!is.null(columns)){
    hierarchy <- hierarchy[columns]
  }
  for(cols in colnames(hierarchy)){
    hierarchy[[cols]]<- as.factor(hierarchy[[cols]])
  }
  
  hierarchy_dt <- as.data.table(hierarchy)
  hierarchy_dt$samples <- rownames(hierarchy_dt) <- rownames(hierarchy)
  
  ncols <- ncol(hierarchy_dt)
  cols <- colnames(hierarchy_dt)[seq_len(ncols-1)]
  order <- rep(1, length(hierarchy_dt)-1)
  hierarchy_dt <- setorderv(hierarchy_dt, cols = cols, order = order)
  hierarchy_df <- as.data.frame(hierarchy_dt)
  rownames(hierarchy_df) <- hierarchy_df$samples
  hierarchy <- hierarchy_df[,cols]
  
  hierarchy <- .rename_clusters(hierarchy)
  
  uniqeness <- .check_unique_parent(hierarchy)
  
  # Create clustree object
  hierarchy_graph <-
    clustree(
      hierarchy ,
      prefix = "cluster",
      prop_filter = 0,
      return = "graph"
    )
  
  if(uniqeness==FALSE){
    # prune the graph with only core edges (this makes it a ~tree)
    graph_df <- as_long_data_frame(hierarchy_graph)
    hierarchy <- .prune_tree(graph_df, hierarchy)
    hierarchy_graph = hierarchy$Clustree_obj
    hierarchy <- DataFrame(hierarchy$Cluster_obj)
  }
  
  # collapses tree if the levels are the same at different resolutions
  collapsed_graph <- .collapse_tree(hierarchy_graph)
  
  cluster_names <-
    unique(vapply(strsplit(collapsed_graph$node, "C"), '[', character(1), 1))
  
  #take only uncollapsed columns
  hierarchy <- hierarchy[, cluster_names]
  
  digits<- sub(pattern = "cluster", replacement = "", x=cluster_names)
  cluster_names <- paste0(digits, ".",cluster_names, sep="")
 
  #renaming nodes from numbers to cluster1C1 cluster1C2 so on..
  for (clusnames in names(hierarchy)) {
    hierarchy[[clusnames]] <-
      paste(clusnames, hierarchy[[clusnames]], sep = 'C')
  }
  
  names(hierarchy)<- cluster_names
 
  samples <- rownames(hierarchy)
  if (is.null(rownames(hierarchy))) {
    samples <- seq_len(nrow(hierarchy))
  }
  
  hierarchy <- cbind(hierarchy, samples)
  hierarchy <- .checkRoot(hierarchy)
  
  new(
    "ClusterHierarchy",
    as(hierarchy, "DFrame")
  )
}

setClassUnion("CharacterOrNull", c("character", "NULL"))

#' TreeIndex class to manage and query hierarchical data
setClass(
  "TreeIndex",
  contains = c("DFrame"),
  representation(
    feature_order = "CharacterOrNull",
    leaf_of_table = "data.table",
    hierarchy_tree = "data.frame",
    node_ids_table = "data.table",
    nodes_table = "data.table"
  )
)

#' create a new TreeIndex object
#' @param hierarchy hierarchy as a data.table
#' @param feature_order order of the tree if different from colnames
#' @importFrom methods new
#' @importFrom data.table data.table
#' @importFrom data.table as.data.table
#' @importFrom data.table setorderv
#' @importFrom data.table melt
#' @importFrom data.table setnames
#' @importFrom S4Vectors DataFrame
#' @import digest
#' @examples 
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' @export
#' @return a `TreeIndex` object
TreeIndex <- function(hierarchy = NULL,
                      feature_order = NULL) {
  if (is.null(hierarchy)) {
    return(
      new(
        "TreeIndex",
        DataFrame(),
        feature_order = feature_order,
        leaf_of_table = data.table(),
        hierarchy_tree = data.frame(),
        node_ids_table = data.table(),
        nodes_table = data.table()
      )
    )
  }

  if (ncol(hierarchy) == 0) {
    return(
      new(
        "TreeIndex",
        DataFrame(hierarchy),
        feature_order = feature_order,
        leaf_of_table = data.table(),
        hierarchy_tree = data.frame(),
        node_ids_table = data.table(),
        nodes_table = data.table()
      )
    )
  }

  if (is.null(feature_order)) {
    feature_order <- colnames(hierarchy)
  }

  hierarchy_tree <-
    .generate_hierarchy_tree(hierarchy, feature_order)
  node_ids_table <-
    .generate_node_ids(hierarchy_tree, feature_order)
  nodes_table <-
    .generate_nodes_table(hierarchy_tree, node_ids_table, feature_order)
  leaf_of_table <-
    .generate_leaf_of_table(hierarchy_tree, node_ids_table, nodes_table, feature_order)

  hierarchy_df <- DataFrame(hierarchy)

  new(
    "TreeIndex",
    hierarchy_df,
    feature_order = feature_order,
    leaf_of_table = leaf_of_table,
    hierarchy_tree = hierarchy_tree,
    node_ids_table = node_ids_table,
    nodes_table = nodes_table
  )
}

#' generate hierarchy tree
#' @param hierarchy hierarchy as a data.table
#' @param feature_order order of the tree if different from colnames
#' @return a data frame object
.generate_hierarchy_tree <- function(hierarchy, feature_order) {
  fd <- hierarchy
  for (i in seq_len(ncol(fd))) {
    fd[, i] = as.character(fd[, i])
  }
  hierarchy <- fd

  replacing_na_obj_fData <- hierarchy[, feature_order]

  nas_replaced <-
    .replaceNAFeatures(replacing_na_obj_fData, feature_order)

  obj_fData <- as.data.table(nas_replaced)
  cols <- feature_order[seq_len(length(feature_order) - 1)]
  order <- rep(1, length(feature_order) - 1)
  ordered_fData <- setorderv(obj_fData, cols = cols, order = order)

  otu_indexes <-
    seq_along(ordered_fData[, get(feature_order[length(feature_order)])])
  ordered_fData <- ordered_fData[, otu_index := otu_indexes]
  ordered_fData_df <- as.data.frame(ordered_fData)

  if (length(unique(ordered_fData_df[, 1])) > 1) {
    allFeatures <- rep("AllFeatures", nrow(ordered_fData_df))
    ordered_fData_df <- cbind(allFeatures, ordered_fData_df)
    feature_order <- unlist(c("allFeatures", feature_order))
  }

  ordered_fData_df
}

#' generate node ids in the tree
#' @param hierarchy_tree hierarchy as a data.table
#' @param feature_order order of the tree if different from colnames
#' @return a data frame object
.generate_node_ids <- function(hierarchy_tree, feature_order) {
  table_node_ids <- hierarchy_tree
  id_list <- vapply(feature_order, function(level) {
    depth <- which(feature_order == level)
    temp_level <-
      data.table(table_node_ids[, c(level, "otu_index")])
    temp_level_count <-
      temp_level[, .(leaf_index = .I[which.min(otu_index)], count = .N), by =
                   eval(level)]

    level_features <- as.character(table_node_ids[[level]])
    for (i in seq_len(nrow(temp_level_count))) {
      row <- temp_level_count[i,]
      if (depth == 1 && i == 1) {
        tid <- paste(depth - 1, 0, sep = "-")
      } else{
        tid <-
          paste(depth - 1, paste(digest(row[, 1], algo = "crc32"), i, sep = ""), sep =
                  "-")
      }
      level_features <-
        replace(level_features, which(level_features == row[[level]]), tid)
    }
    level_features
  }, character(nrow(table_node_ids)))
  
  node_ids_dt <- as.data.table(id_list)
  node_ids_dt$otu_index <- as.character(table_node_ids$otu_index)

  node_ids_table <- node_ids_dt
}

#' generate nodes table tree
#' @param hierarchy_tree hierarchy as a data.table
#' @param node_ids_table node ids
#' @param feature_order order of the tree if different from colnames
#' @return a data frame object
.generate_nodes_table <-
  function(hierarchy_tree,
           node_ids_table,
           feature_order) {
    print(feature_order)
    lineage_DF <- as.data.frame(node_ids_table)
    lineage_table <- node_ids_table
    lineage_DF[, feature_order[1]] <-
      lineage_table[, get(feature_order[1])]

    for (i in seq(2, length(feature_order))) {
      lineage_DF[, feature_order[i]] <-
        paste(lineage_DF[, feature_order[i - 1]], lineage_table[, get(feature_order[i])], sep =
                ",")
    }
    lineage_DT <- as.data.table(lineage_DF)

    root_parents <-
      rep("None", length(node_ids_table[, get(feature_order[1])]))
    nodes_tab <-
      data.frame(
        id = node_ids_table[, get(feature_order[1])],
        parent = root_parents,
        lineage = node_ids_table[, get(feature_order[1])],
        node_label = hierarchy_tree[, 1],
        level = rep(1, length(hierarchy_tree[, 1]))
      )

    for (i in seq(2, length(feature_order))) {
      temp_nodes_tab <-
        data.frame(
          id = node_ids_table[, get(feature_order[i])],
          parent = node_ids_table[, get(feature_order[i -
                                                        1])],
          lineage = lineage_DT[, get(feature_order[i])],
          node_label = hierarchy_tree[, i],
          level = rep(i, length(hierarchy_tree[, i]))
        )

      nodes_tab <-
        rbind(nodes_tab[rownames(unique(nodes_tab[, c("id", "parent")])),], temp_nodes_tab[rownames(unique(temp_nodes_tab[, c("id", "parent")])),])
    }

    ret_table <- as.data.table(nodes_tab)
    ret_table <- ret_table[, id := as.character(id)]
    ret_table <- ret_table[, parent := as.character(parent)]
    ret_table <- ret_table[, lineage := as.character(lineage)]
    ret_table <- ret_table[, node_label := as.character(node_label)]
    ret_table <- ret_table[, level := as.integer(level)]

    ret_table <- ret_table[order(parent)]
    parent_list <- ret_table[, parent]
    orders <- rep(1, length(parent_list))

    for (j in seq(2, length(parent_list))) {
      if (parent_list[j] == parent_list[j - 1]) {
        orders[j] = orders[j - 1] + 1
      }
    }
    ret_table[, order := orders]

    ret_table
  }

#' generate leaf of table
#' @param hierarchy_tree hierarchy as a data.table
#' @param node_ids_table node ids
#' @param nodes_table nodes table
#' @param feature_order order of the tree if different from colnames
#' @return a data frame object
.generate_leaf_of_table <-
  function(hierarchy_tree,
           node_ids_table,
           nodes_table,
           feature_order) {
    temp_hiearchy_DT <- as.data.table(hierarchy_tree)
    num_features <- length(feature_order)
    hiearchy_cols <- colnames(hierarchy_tree)

    melt_res <-
      melt(
        temp_hiearchy_DT,
        id.vars = c(feature_order[num_features], "otu_index"),
        measure.vars = c(hiearchy_cols[seq_len(length(hiearchy_cols) -
                                            1)])
      )
    label_table <- melt_res[, c(1, 2, 4)]
    setnames(label_table, c("leaf", "otu_index", "node_label"))

    label_table <- label_table[, leaf := as.character(leaf)]
    label_table <-
      label_table[, otu_index := as.character(otu_index)]

    lineage_DF <- as.data.frame(node_ids_table)
    lineage_table <- node_ids_table
    lineage_DF[, feature_order[1]] <-
      lineage_table[, get(feature_order[1])]

    for (i in seq(2, length(feature_order))) {
      lineage_DF[, feature_order[i]] <-
        paste(lineage_DF[, feature_order[i - 1]], lineage_table[, get(feature_order[i])], sep =
                ",")
    }
    lineage_DT <- as.data.table(lineage_DF)

    melt_res_lineage <-
      melt(
        lineage_DT,
        id.vars = c(feature_order[num_features], "otu_index"),
        measure.vars = c(hiearchy_cols[seq_len(length(hiearchy_cols) - 1)])
      )

    lineage_leaf_of_table <- unique(melt_res_lineage[, c(2, 4)])
    setnames(lineage_leaf_of_table, c("otu_index", "lineage"))

    lineage_leaf_of_table <-
      lineage_leaf_of_table[, otu_index := as.character(otu_index)]

    lineage_df <- as.data.frame(lineage_leaf_of_table)
    leaf_node_label <-
      as.data.frame(label_table)[, c("leaf", "node_label")]

    ret_table <- as.data.table(cbind(lineage_df, leaf_node_label))

    leaf_of_table <- ret_table

    leaf_of_table <-
      merge(unique(nodes_table[, mget(c("lineage", "id"))]),
            unique(leaf_of_table) , by = "lineage")
    leaf_of_table[, id := as.character(id)]
  }

#' replace if there are NA's in the hierarchy
#' @param replacing_na_obj_fData hierarchy data table
#' @param feature_order order of the tree if different from colnames
#' @return a data frame object
.replaceNAFeatures = function(replacing_na_obj_fData, feature_order) {
  for (i in seq_len(length(feature_order))) {
    na_indices <-
      which(is.na(replacing_na_obj_fData[, feature_order[i]]))
    for (j in seq_len(length(na_indices))) {
      if (i > 1) {
        replacing_na_obj_fData[, feature_order[i]][na_indices[j]] <-
          paste("Not_Annotated",
                feature_order[i],
                replacing_na_obj_fData[, feature_order[1]][na_indices[j]],
                sep = "_")
      } else {
        replacing_na_obj_fData[, feature_order[i]][na_indices[j]] <-
          paste("Not_Annotated", feature_order[i], sep = "_")
      }
    }
    na_indices <-
      which(replacing_na_obj_fData[, feature_order[i]] == "NA")
    for (j in seq_len(length(na_indices))) {
      if (i > 1) {
        replacing_na_obj_fData[, feature_order[i]][na_indices[j]] <-
          paste("Not_Annotated",
                feature_order[i],
                replacing_na_obj_fData[, feature_order[1]][na_indices[j]],
                sep = "_")
      } else {
        replacing_na_obj_fData[, feature_order[i]][na_indices[j]] <-
          paste("Not_Annotated", feature_order[i], sep = "_")
      }
    }
    
    null_indices <-
      which(replacing_na_obj_fData[, feature_order[i]] == "NULL")
    for (j in seq_len(length(null_indices))) {
      if (i > 1) {
        replacing_na_obj_fData[, feature_order[i]][null_indices[j]] <-
          paste("Not_Annotated",
                feature_order[i],
                replacing_na_obj_fData[, feature_order[1]][null_indices[j]],
                sep = "_")
      } else{
        replacing_na_obj_fData[, feature_order[i]][null_indices[j]] <-
          paste("Not_Annotated", feature_order[i], sep = "_")
      }
    }
  }

  replacing_na_obj_fData
}

#' Subset TreeIndex
#' @param x TreeIndex to subset
#' @param i,j indices to subset or keep
#' @param ... other params to dataframe subset method
#' @param drop drop the dimensions of the object. defaults to FALSE
#' @importFrom methods callNextMethod
#' @return a `TreeIndex` subset object
#' 
#' @rdname TreeIndex-methods
setMethod("[", "TreeIndex",
          function(x, i, j, ..., drop = FALSE) {
            # obj <- callNextMethod()
            new_hierarchy_tree <-
              unique(x@hierarchy_tree[i, j, ..., drop = drop])
            new_feature_order <- colnames(new_hierarchy_tree)

            TreeIndex(hierarchy = new_hierarchy_tree,
                      feature_order = new_feature_order)
          })

#' Generic method to get nodes at a tree level
#' @param x object
#' @param ... other parameters
#' @examples 
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' getNodes(tree)
#' @return a generic
#' 
#' @rdname TreeIndex-methods
setGeneric("getNodes", signature = "x",
           function(x, ...)
             standardGeneric("getNodes"))

#' Method to get nodes at a tree level
#' @param x a TreeIndex object
#' @param selectedLevel tree level to select nodes from
#' @examples 
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' getNodes(tree)
#' @export
#' @return levels at node cut
#' 
#' @rdname TreeIndex-methods
setMethod("getNodes", "TreeIndex",
          function(x,
                   selectedLevel = NULL) {

            if (is.null(selectedLevel)) {
              return(x@nodes_table[,c("id", "lineage", "node_label", "level")])
            }
            nodes_at_level <-
              x@nodes_table[level == selectedLevel,]
            nodes_at_level_ids <- nodes_at_level[, id]
            unique(data.frame(ids = nodes_at_level_ids, names = nodes_at_level$node_label))
          })

#' Generic method for possible node states
#' @param x object
#' @return node state
#' @rdname TreeIndex-methods
setGeneric("getNodeStates", signature = "x",
           function(x)
             standardGeneric("getNodeStates"))

#' Method to get possible node states
#' a node state is 0 if removed, 1 if expanded to show children &
#' 2 if counts are aggregated to the node
#' @param x object
#' @return node states
#' @rdname TreeIndex-methods
setMethod("getNodeStates", "TreeIndex",
          function(x) {
            return(list(
              "removed" = 0,
              "expanded" = 1,
              "aggregate" = 2
            ))
          })


#' Generic method to split the tree
#' @param x object
#' @param ... other parameters
#' @examples 
#' \donttest{ 
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' splitAt(tree)
#' }
#' @return a generic
#' @rdname TreeIndex-methods
setGeneric("splitAt", signature = "x",
           function(x, ...)
             standardGeneric("splitAt"))

#' splitAt divides the TreeIndex into groups defined by the level,
#' node selections and filters(start, end)
#' @param x TreeIndex object
#' @param selectedLevel tree level to select nodes from
#' @param selectedNodes used to set states on individual nodes to define a cut on the tree
#' @param start,end indices to filter nodes by
#' @param format return format can be one of "list" or "TreeIndex"
#' @importFrom stats na.omit
#' @importFrom methods is
#' @examples 
#' \donttest{ 
#' library(metagenomeSeq)
#' data(mouseData)
#' counts <- MRcounts(mouseData)
#' hierarchy <- fData(mouseData)
#' tree <- TreeIndex(hierarchy)
#' splitAt(tree)
#' }
#' @export
#' @return a `TreeIndex` object or type set in format
#' @rdname TreeIndex-methods
setMethod("splitAt", "TreeIndex",
          function(x,
                   selectedLevel = 3,
                   selectedNodes = NULL,
                   start = 1,
                   end = NULL,
                   format = "list") {
            if (is.null(end)) {
              end <- nrow(x)
            }

            nodes_at_level <-
              x@nodes_table[level == selectedLevel,]
            nodes_at_level_ids <- nodes_at_level[, id]

            if (!is.null(selectedNodes) &&
                !(length(selectedNodes) == 0)) {
              nodes_at_level_selections <- rep(2, length(nodes_at_level_ids))
              names(nodes_at_level_selections) <- nodes_at_level_ids
              selectedNodes <-
                c(selectedNodes, nodes_at_level_selections)

              expand_selections <- which(selectedNodes == 1)
              if (!is.null(expand_selections) &&
                  length(expand_selections) > 0) {
                expand_selection_indices = which(names(selectedNodes) %in% names(expand_selections))
                selectedNodes <-
                  selectedNodes[-expand_selection_indices]
              }

              expanded_children <-
                x@nodes_table[parent %in% names(expand_selections), id]

              child_lineage <-
                x@nodes_table[id %in% names(selectedNodes), ]
              remove_selections <- which(selectedNodes == 0)
              if (length(remove_selections) > 0) {
                kept_nodes <-
                  child_lineage[!grepl(paste(paste(
                    names(remove_selections), collapse = ",|"
                  ), ",", sep = ""), lineage), ]
                kept_nodes <-
                  kept_nodes[!(id %in% names(remove_selections)), ]
              } else {
                kept_nodes <- child_lineage
              }

              agg_selections <- which(selectedNodes == 2)
              if (length(agg_selections) > 0) {
                kept_nodes <-
                  kept_nodes[!grepl(paste(paste(names(
                    agg_selections
                  ), collapse = ",|"), ",", sep = ""), lineage), ]
              }
              kept_nodes <- as.character(kept_nodes[, id])
              nodes_at_level <-
                x@nodes_table[id %in% c(kept_nodes, expanded_children), ]
            }

            leaf_order_table <-
              as.data.table(x@hierarchy_tree[, c(x@feature_order[length(x@feature_order)], "otu_index")])
            setnames(leaf_order_table, c("leaf", "otu_index"))
            leaf_order_table <-
              leaf_order_table[, leaf := as.character(leaf)]
            leaf_order_table <-
              leaf_order_table[otu_index >= start &
                                 otu_index <= end]

            leaf_indices <-
              merge(leaf_order_table,
                    merge(nodes_at_level, x@leaf_of_table, by = "id"),
                    by = "leaf")
            setorderv(leaf_indices, "otu_index.x")
            leaf_indices$lineage.y <- NULL
            leaf_indices$otu_index.y <- NULL
            leaf_indices$node_label.y <- NULL
            colnames(leaf_indices) <-
              c("leaf",
                "otu_index",
                "id",
                "parent",
                "lineage",
                "node_label",
                "level",
                "order")

            if (format == "aggTable") {
              return(leaf_indices)
            }
            else if (format == "TreeIndex") {
              toLevel <- selectedLevel
              new_feature_order <- x@feature_order[seq_len(toLevel)]
              new_hierarchy_tree <-
                unique(x@hierarchy_tree[start:end, new_feature_order])
              new_hierarchy_tree <- na.omit(new_hierarchy_tree)
              groups <-
                unique(leaf_indices[, .(
                  indices = paste0(otu_index, collapse = ","),
                  leaf_nodes = paste0(leaf, collapse = ",")
                ), by = .(id, parent, lineage, node_label, level, order)])
              rownames(new_hierarchy_tree) <- make.names(groups$node_label, unique = TRUE)

              # new_hierarchy_tree <- as.data.frame(unique(as.data.table(new_hierarchy_tree), by=new_feature_order[toLevel]))
              newTreeIndex <-
                TreeIndex(hierarchy = new_hierarchy_tree,
                          feature_order = new_feature_order)
              return(newTreeIndex)
            }
            else if (format == "dataframe") {
              groups <-
                leaf_indices[, .(
                  indices = paste0(otu_index, collapse = ","),
                  leaf_nodes = paste0(leaf, collapse = ",")
                ), by = .(id, parent, lineage, node_label, level, order)]
              return(groups)
            }
            else if (format == "list") {
              groups <-
                unique(leaf_indices[, .(
                  indices = paste0(otu_index, collapse = ","),
                  leaf_nodes = paste0(leaf, collapse = ",")
                ), by = .(id, parent, lineage, node_label, level, order)])
              nodes <- as.list(groups$indices)
              nodes_exp <- lapply(nodes, function(nl) {
                as.integer(strsplit(nl, ",")[[1]])
              })
              names(nodes_exp) <- make.names(groups$node_label, unique = TRUE)
              return(nodes_exp)
            }
          })

#' Show the TreeIndex object
#' @importFrom methods show
#' @param object TreeIndex object
#' @return object description of the `TreeIndex` object
#' @rdname TreeIndex-methods
setMethod("show", "TreeIndex", function(object) {
  cat(
    "Tree Index",
    "with height:",
    length(object@feature_order),
    "\n Tree levels:",
    paste(object@feature_order, collapse = " -> "),
    "\n Leaf nodes:",
    nrow(object),
    sep = " "
  )
})

setAs("DataFrame", "TreeIndex", function(from) {
  as.data.frame(from)
})

---
title: "Interactively explore and visualize Single Cell RNA seq data"
author: "Jayaram Kancherla, Kazi Tasnim Zinat, Héctor Corrada Bravo"
date: "`r Sys.Date()`"
output: BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{Explore Data using scTreeViz}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

[`scTreeViz`](https://github.com/HCBravoLab/scTreeViz) is a package for interactive visualization and exploration of Single Cell RNA sequencing data. `scTreeViz` provides methods for exploring hierarchical features (eg. clusters in single cell at different resolutions or taxonomic hierarchy in single cell datasets), while supporting other useful data visualization charts like heatmaps for expression and scatter plots for dimensionality reductions like UMAP or TSNE.

## Loading required packages

```{r load-packages, message=FALSE, warning=FALSE}
library(scTreeViz)
library(Seurat)
library(SC3)
library(scran)
library(scater)
library(clustree)
library(igraph)
library(scRNAseq)
```

# Preparing Datasets

The first step in using the `scTreeViz` package is to wrap datasets into `TreeViz` objects. The `TreeViz` class extends `SummarizedExperiment` and provides various methods to interactively perform various operations on the underlying hierarchy and count or expression matrices. In this section, we show various ways to generate a `TreeViz` object either from existing Single Cell packages (SingleCellExperiment or Seurat) or from a raw count matrix and cluster hierarchy.

## From `SingleCellExperiment`

A number of Single cell datasets are available as `SingleCellExperiment` objects through the `scRNAseq` package, for this usecase, we use `LunSpikeInData` dataset. In addition, we calculate the dimensionality reductions; UMAP, TSNE and PCA from the functions provided in `scater` package. 

```{r, results='hide', warning=FALSE, error=FALSE, message=FALSE}
# load dataset
sce<- ZeiselBrainData()
# Normalization
sce <- logNormCounts(sce)
# calculate umap and tsne
sce <- runUMAP(sce)
sce<- runTSNE(sce)
sce<- runPCA(sce)
```

We provide `createFromSCE` function to create a `TreeViz` object from `SingleCellExperiment` object. Here, the workflow works in two ways:

1. If no cluster information is available in the `colData` of the `SingleCellExperiment` object, we create clusters at different resolutions using the `WalkTrap` algorithm by calling an internal function `generate_walktrap_hierarchy` and use this cluster information for visualization.

```{r, warning=FALSE, error=FALSE, message=FALSE}
treeViz <- createFromSCE(sce, reduced_dim = c("UMAP","PCA","TSNE"))
plot(treeViz)
```

2. If cluster information is provided in the `colData` of the object, then the user should set the flag parameter `check_coldata` to `TRUE` and provide prefix for the columns where cluster information is stored.

```{r, eval=FALSE, warning=FALSE, error=FALSE, message=FALSE}
# Forming clusters
set.seed(1000)
for (i in  seq(10)) {
  clust.kmeans <- kmeans(reducedDim(sce, "TSNE"), centers = i)
  sce[[paste0("clust", i)]] <- factor(clust.kmeans$cluster)
}

treeViz<- createFromSCE(sce, check_coldata = TRUE, col_regex = "clust")
plot(treeViz)
```

Note: In both cases the user needs to provide the name of dimensionality reductions present in the object as a parameter.

## From `Seurat`

We use the dataset `pbmc_small` available through Seurat to create a `TreeViz` object.  

```{r, echo=TRUE, results='hide', warning=FALSE, error=FALSE, message=FALSE}
data(pbmc_small)
pbmc <- pbmc_small
```

We then preprocess the data and find clusters at different resolutions.

```{r, echo=TRUE, results='hide', warning=FALSE, error=FALSE, message=FALSE}
pbmc[["percent.mt"]] <- PercentageFeatureSet(pbmc, pattern = "^MT-")
pbmc <- NormalizeData(pbmc)
all.genes <- rownames(pbmc)
pbmc <- ScaleData(pbmc, vars.to.regress = "percent.mt")
pbmc <- FindVariableFeatures(object = pbmc)
pbmc <- RunPCA(pbmc, features = VariableFeatures(object = pbmc))
pbmc <- FindNeighbors(pbmc, dims = 1:10)
pbmc <- FindClusters(pbmc, resolution = c(0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0), print.output = 0, save.SNN = TRUE)
pbmc
```

The measurements for dimensionality reduction methods we want to visualize are also added to the object via native functions in `Seurat`. Since `PCA` is already added, we calculate `TSNE` and `UMAP`
```{r, echo=TRUE, results='hide', warning=FALSE, error=FALSE, message=FALSE}
# pbmc<- RunTSNE(pbmc)
pbmc<- RunUMAP(pbmc, dims=1:3)
Reductions(pbmc)
```

We use the `createFromSeurat` function to create a `TreeViz` object from `Seurat` object. In addition the object, we pass the name of dimensionality reductions present in the object as a paramter in vector format to indicate these measurements should be added to `treeviz` for visualization. If the mentioned reduced dimension is not present it would simply be ignored.
```{r, echo=TRUE,  warning=FALSE, error=FALSE, message=FALSE}
treeViz<- createFromSeurat(pbmc, check_metadata = TRUE, reduced_dim = c("umap","pca","tsne"))
plot(treeViz)
```

## Create TreeViz from count matrix and Cluster hierarchy

```{r, results='hide', warning=FALSE, error=FALSE, message=FALSE}
n=64
# create a hierarchy
df<- data.frame(cluster0=rep(1,n))
for(i in seq(1,5)){
  df[[paste0("cluster",i)]]<- rep(seq(1:(2**i)),each=ceiling(n/(2**i)),len=n)
}

# generate a count matrix
counts <- matrix(rpois(6400, lambda = 10), ncol=n, nrow=100)
colnames(counts)<- seq(1:64)
# create a `TreeViz` object
treeViz <- createTreeViz(df, counts)
plot(treeViz)
```


## Start the TreeViz App (using hosted app)

Start the App from the `treeViz` object we created. This adds a `facetZoom` to navigate the cluster hierarchy, a heatmap of the top `n` most variable genes from the dataset, where 'n' is selected by the user and one scatter plot for each of the reduced dimensions. 
 
```{r, eval=FALSE, echo=TRUE}
app <- startTreeviz(treeViz, top_genes = 500)
```
![Cell 416B dataset](images/cell416B.png)

![pbmc dataset](images/Treeviz.png)

Users can also use the interface to explore the same dataset using different visualizations available through Epiviz.

## Visualize gene expression across clusters

Users can also add Gene Box plots using either the frontend application, or from R session. In the following example, we visualize the 5th, 50th and 500th most variable gene as Box plots
![visualizing expression of a gene across clusters](images/Boxplot.png)

### Adding Gene Box Plots via UI

Users need to select  `Add Visualization -> Gene Box PLot` option from menu and then select the desired gene using the search pane in the appeared dialogue box

![Selecting a gene to visualize](images/dest_boxplot_dialogue.png)

### Adding Gene Box Plots via R Session

Users can also select the gene from R session by using the `plotGene` command followed by Gene name.
```{r, eval=FALSE}
app$plotGene(gene="AIF1")
```

## Stop App

After exploring the dataset, this command the websocket connection. 

```{r, eval=FALSE, echo=TRUE}
app$stop_app()
```

## Session

```{r}
sessionInfo()

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EpivizTreeData-class.R
\docType{class}
\name{EpivizTreeData-class}
\alias{EpivizTreeData-class}
\alias{EpivizTreeData}
\title{Data container for MRexperiment objects}
\description{
Used to serve hierarchical data (used in e.g., icicle plots and heatmaps).
}
\section{Methods}{

\describe{
\item{\code{df_to_tree(root, df)}}{Helper function to recursively build nested response for getHierarchy
\describe{
\item{root}{Root of subtree}
\item{df}{data.frame containing children to process}
}}

\item{\code{get_default_chart_type()}}{Get name of default chart type for this data type}

\item{\code{get_measurements()}}{Get description of measurements served by this object}

\item{\code{getCombined(
  measurements = NULL,
  seqName,
  start = 1,
  end = 1000,
  order = NULL,
  nodeSelection = NULL,
  selectedLevels = NULL
)}}{Return the counts aggregated to selected nodes for the given samples
\describe{
\item{measurements}{Samples to get counts for}
\item{seqName}{name of datasource}
\item{start}{Start of feature range to query}
\item{end}{End of feature range to query}
\item{order}{Ordering of nodes}
\item{nodeSelection}{Node-id and selectionType pairs}
\item{selectedLevels}{Current aggregation level}
}}

\item{\code{getHierarchy(nodeId = NULL)}}{Retrieve feature hierarchy information for subtree with specified root
\describe{
\item{nodeId}{Feature identifier with level info}
}}

\item{\code{getReducedDim(method = NULL, gene = NULL)}}{Compute PCA over all features for given samples
\describe{
\item{method}{which dimension to access}
\item{gene}{send expression of a gene back with the dimensions}
}}

\item{\code{getRows(
  measurements = NULL,
  start = 1,
  end = 1000,
  selectedLevels = 3,
  selections = NULL
)}}{Return the sample annotation and features within the specified range and level for a given sample and features
\describe{
\item{measurements}{Samples to retrieve for}
\item{start}{Start of feature range to query}
\item{end}{End of feature range to query}
\item{selections}{Node-id and selectionType pairs}
\item{selectedLevels}{Current aggregation level}
}}

\item{\code{propagateHierarchyChanges(
  selection = NULL,
  order = NULL,
  selectedLevels = NULL,
  request_with_labels = FALSE
)}}{Update internal state for hierarchy
\describe{
\item{selection}{Node-id and selectionType pairs}
\item{order}{Ordering of features}
\item{selectedLevels}{Current aggregation level}
\item{request_with_labels}{For handling requests using fData entries from MRexperiment}
}}

\item{\code{row_to_dict(row)}}{Helper function to format each node entry for getHierarchy response
\describe{
\item{row}{Information for current node.}
}}

\item{\code{searchTaxonomy(query = NULL, max_results = 15)}}{Return list of features matching a text-based query
\describe{
\item{query}{String of feature for which to search}
\item{max_results}{Maximum results to return}
}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper-functions.R
\name{createFromSeurat}
\alias{createFromSeurat}
\title{Creates a `TreeViz` object from `Seurat`}
\usage{
createFromSeurat(
  object,
  check_metadata = FALSE,
  col_regex = "*snn*",
  columns = NULL,
  reduced_dim = c("TSNE")
)
}
\arguments{
\item{object}{`Seurat` class containing cluster information at different resolutions}

\item{check_metadata}{whether to metaData of `Seurat` object for cluster information or not}

\item{col_regex}{common regular expression shared across all columns with cluster information}

\item{columns}{vector containing columns with cluster information}

\item{reduced_dim}{Vector of Dimensionality reduction information provided in `Seurat` object to be added in `TreeViz` (if exists)}
}
\value{
`TreeViz` Object
}
\description{
Creates a `TreeViz` object from `Seurat`
}
\examples{
library(Seurat)
data(pbmc_small)
pbmc <- pbmc_small
treeviz<- createFromSeurat(pbmc, check_metadata = TRUE, reduced_dim = c("pca","tsne"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeVizApp-class.R
\docType{class}
\name{TreeVizApp-class}
\alias{TreeVizApp-class}
\alias{TreeVizApp}
\title{Class managing connection to metaviz application.}
\description{
Class managing connection to metaviz application.
}
\section{Methods}{

\describe{
\item{\code{plotGene(gene = NULL, datasource_name = "SCRNA_1")}}{Plot a bar plot for a gene across cell types
\describe{
\item{gene}{gene to extract expression values}
\item{datasource_name}{object to extract from (automatically selected)}
}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-class.R
\docType{class}
\name{TreeIndex-class}
\alias{TreeIndex-class}
\title{TreeIndex class to manage and query hierarchical data}
\description{
TreeIndex class to manage and query hierarchical data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper-functions.R
\name{set_gene_list}
\alias{set_gene_list}
\title{Sets gene list for visualization}
\usage{
set_gene_list(treeviz, genes)
}
\arguments{
\item{treeviz}{TreeViz object}

\item{genes}{list of genes to use}
}
\value{
TreeViz object set with gene list
}
\description{
Sets gene list for visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-methods.R
\name{[,TreeIndex,ANY,ANY,ANY-method}
\alias{[,TreeIndex,ANY,ANY,ANY-method}
\alias{getNodes}
\alias{getNodes,TreeIndex-method}
\alias{getNodeStates}
\alias{getNodeStates,TreeIndex-method}
\alias{splitAt}
\alias{splitAt,TreeIndex-method}
\alias{show,TreeIndex-method}
\title{Subset TreeIndex}
\usage{
\S4method{[}{TreeIndex,ANY,ANY,ANY}(x, i, j, ..., drop = FALSE)

getNodes(x, ...)

\S4method{getNodes}{TreeIndex}(x, selectedLevel = NULL)

getNodeStates(x)

\S4method{getNodeStates}{TreeIndex}(x)

splitAt(x, ...)

\S4method{splitAt}{TreeIndex}(
  x,
  selectedLevel = 3,
  selectedNodes = NULL,
  start = 1,
  end = NULL,
  format = "list"
)

\S4method{show}{TreeIndex}(object)
}
\arguments{
\item{x}{TreeIndex object}

\item{i, j}{indices to subset or keep}

\item{...}{other parameters}

\item{drop}{drop the dimensions of the object. defaults to FALSE}

\item{selectedLevel}{tree level to select nodes from}

\item{selectedNodes}{used to set states on individual nodes to define a cut on the tree}

\item{start, end}{indices to filter nodes by}

\item{format}{return format can be one of "list" or "TreeIndex"}

\item{object}{TreeIndex object}
}
\value{
a `TreeIndex` subset object

a generic

levels at node cut

node state

node states

a generic

a `TreeIndex` object or type set in format

object description of the `TreeIndex` object
}
\description{
Subset TreeIndex

Generic method to get nodes at a tree level

Method to get nodes at a tree level

Generic method for possible node states

Method to get possible node states
a node state is 0 if removed, 1 if expanded to show children &
2 if counts are aggregated to the node

Generic method to split the tree

splitAt divides the TreeIndex into groups defined by the level,
node selections and filters(start, end)

Show the TreeIndex object
}
\examples{
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
getNodes(tree)
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
getNodes(tree)
\donttest{ 
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
splitAt(tree)
}
\donttest{ 
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
splitAt(tree)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-class.R
\name{.generate_nodes_table}
\alias{.generate_nodes_table}
\title{generate nodes table tree}
\usage{
.generate_nodes_table(hierarchy_tree, node_ids_table, feature_order)
}
\arguments{
\item{hierarchy_tree}{hierarchy as a data.table}

\item{node_ids_table}{node ids}

\item{feature_order}{order of the tree if different from colnames}
}
\value{
a data frame object
}
\description{
generate nodes table tree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeViz-class.R
\name{TreeViz}
\alias{TreeViz}
\title{The TreeViz class.}
\usage{
TreeViz(assays = SimpleList(), rowData = NULL, colData = NULL, ...)
}
\arguments{
\item{assays}{simple list of counts}

\item{rowData}{rowData}

\item{colData}{colData}

\item{...}{other parameters for SummarizedExperiment}
}
\value{
a `TreeViz` object
}
\description{
SummarizedExperiment-like class for datasets that have hierarchies on either rowData or colData.
For microbiome data, rowData is a tree hierarchy
For single cell data, colData is a tree hierarchy
}
\examples{
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-class.R
\name{.generate_leaf_of_table}
\alias{.generate_leaf_of_table}
\title{generate leaf of table}
\usage{
.generate_leaf_of_table(
  hierarchy_tree,
  node_ids_table,
  nodes_table,
  feature_order
)
}
\arguments{
\item{hierarchy_tree}{hierarchy as a data.table}

\item{node_ids_table}{node ids}

\item{nodes_table}{nodes table}

\item{feature_order}{order of the tree if different from colnames}
}
\value{
a data frame object
}
\description{
generate leaf of table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper-functions.R
\name{createTreeViz}
\alias{createTreeViz}
\title{Creates `TreeViz` object from hierarchy and count matrix}
\usage{
createTreeViz(clusters, counts)
}
\arguments{
\item{clusters}{`ClusterHierarchy` object or a dataframe containing cluster information at different resolutions}

\item{counts}{matrix Dense or sparse matrix containing the count matrix}
}
\value{
`TreeViz`` Object
}
\description{
Provided with a count matrix and a dataframe or `ClusterHierarchy` object, this module 
runs the necessary checks on the dataframe and tries to convert it to a tree by making necessary changes.
Returns the `TreeViz` object if a tree is successfully generated from dataframe, throws error otherwise
}
\examples{
n=64
# create a hierarchy
df<- data.frame(cluster0=rep(1,n))
for(i in seq_len(5)){
  df[[paste0("cluster",i)]]<- rep(seq(1:(2**i)),each=ceiling(n/(2**i)),len=n)
}
# generate a count matrix
counts <- matrix(rpois(6400, lambda = 10), ncol=n, nrow=100)
colnames(counts)<- seq_len(64)
# create a `TreeViz` object
treeViz <- createTreeViz(df, counts)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ClusterHierarchy-class.R
\docType{class}
\name{ClusterHierarchy-class}
\alias{ClusterHierarchy-class}
\title{ClusterHierarchy class to manage treeviz cluster data}
\description{
ClusterHierarchy class to manage treeviz cluster data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-class.R
\name{.replaceNAFeatures}
\alias{.replaceNAFeatures}
\title{replace if there are NA's in the hierarchy}
\usage{
.replaceNAFeatures(replacing_na_obj_fData, feature_order)
}
\arguments{
\item{replacing_na_obj_fData}{hierarchy data table}

\item{feature_order}{order of the tree if different from colnames}
}
\value{
a data frame object
}
\description{
replace if there are NA's in the hierarchy
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeViz-methods.R
\name{show,TreeViz-method}
\alias{show,TreeViz-method}
\alias{aggregateTree}
\alias{aggregateTree,TreeViz-method}
\alias{register,TreeViz-method}
\alias{plot,TreeViz,ANY-method}
\title{show object}
\usage{
\S4method{show}{TreeViz}(object)

aggregateTree(x, ...)

\S4method{aggregateTree}{TreeViz}(
  x,
  selectedLevel = 3,
  selectedNodes = NULL,
  aggFun = colSums,
  start = 1,
  end = NULL,
  by = "row",
  format = "TreeViz"
)

\S4method{register}{TreeViz}(object, tree = "row", columns = NULL, ...)

\S4method{plot}{TreeViz,ANY}(x, y)
}
\arguments{
\item{object}{The object to register to data server}

\item{x}{treeviz object}

\item{...}{Additional arguments passed to object constructors}

\item{selectedLevel}{level to select nodes from}

\item{selectedNodes}{used to set states on individual nodes to define a cut on the tree}

\item{aggFun}{aggregate function to use, by default colSums if by="row", rowSums if by="col"}

\item{start, end}{indices to filter nodes}

\item{by}{"row" to aggregate the TreeIndex on rowData, "col" to aggregate TreeIndex on colData}

\item{format}{return format can be one of "counts" or "TreeViz"}

\item{tree}{Is tree over rows or columns of the object (default: "row")}

\item{columns}{Name of columns containing data to register}

\item{y}{none}
}
\value{
describe a TreeIndex object

a generic

a Treeviz object or type specified by format

An \code{\link{EpivizTreeData-class}} object

Dataframe containing cluster information at different resolutions
}
\description{
show object

Method to aggregate a TreeViz object

Method to aggregate a TreeViz object

Generic method to register data to the epiviz data server

plot tree from TreeViz
}
\section{Functions}{
\itemize{
\item \code{show,TreeViz-method}: 

\item \code{aggregateTree}: 

\item \code{aggregateTree,TreeViz-method}: 

\item \code{register,TreeViz-method}: 

\item \code{plot,TreeViz,ANY-method}: 
}}

\examples{
\donttest{ 
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
aggregateTree(mbiome)
}
\donttest{ 
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
aggregateTree(mbiome)
}
\donttest{
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
mbiome <- TreeViz(SimpleList(counts=counts), rowData=tree)
plot(mbiome)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ClusterHierarchy-class.R
\name{ClusterHierarchy}
\alias{ClusterHierarchy}
\title{Creates a new ClusterHierarchy object.}
\usage{
ClusterHierarchy(hierarchy, col_regex = NULL, columns = NULL)
}
\arguments{
\item{hierarchy}{hierarchy as a dataFrame}

\item{col_regex}{Regular Expression for choosing columns}

\item{columns}{Vector containing list of columns to choose from with ordering}
}
\value{
`ClusterHierarchy`` return an object of class ClusterHierarchy containing cluster information that ensures a valid dataframe for treeviz input
}
\description{
Works as a validation check for multiple issues user passed dataframe might have.
For example, multiple root nodes, incompatible naming, multiple parents of a single node, etc
This function performs all this checks and tries to resolve the issues by making changes in cluster assignment
User can give either
col_regex or  columns option to filter the columns or specify 
the column order
}
\examples{
n=64
# create a hierarchy
df<- data.frame(cluster0=rep(1,n))
for(i in seq(1,5)){
  df[[paste0("cluster",i)]]<- rep(seq(1:(2**i)),each=ceiling(n/(2**i)),len=n)
}
clus_hier<-ClusterHierarchy(df, col_regex = "clus")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper-functions.R
\name{createFromSCE}
\alias{createFromSCE}
\title{Creates a `TreeViz`` object from `SingleCellExperiment`. Generates
clusters based on Walktrap algorithm if no default is provided}
\usage{
createFromSCE(
  object,
  check_coldata = FALSE,
  col_regex = NULL,
  columns = NULL,
  reduced_dim = c("TSNE")
)
}
\arguments{
\item{object}{`SingleCellExperiment` object to be visualized}

\item{check_coldata}{whether to colData of `SingeCellExperiment` object for cluster information or not}

\item{col_regex}{common regular expression shared across all columns with cluster information}

\item{columns}{vector containing columns with cluster information}

\item{reduced_dim}{Vector of Dimensionality reduction information provided in `SingeCellExperiment` object to be added in `TreeViz` (if exists)}
}
\value{
`TreeViz` Object
}
\description{
Creates a `TreeViz`` object from `SingleCellExperiment`. Generates
clusters based on Walktrap algorithm if no default is provided
}
\examples{
library(SingleCellExperiment)
library(scater)
sce <- mockSCE()
sce <- logNormCounts(sce)
sce <- runTSNE(sce)
sce <- runUMAP(sce)
set.seed(1000)
for (i in  seq_len(5)) {
clust.kmeans <- kmeans(reducedDim(sce, "TSNE"), centers = i)
sce[[paste0("clust", i)]] <- factor(clust.kmeans$cluster)
}
treeviz <-createFromSCE(sce, check_coldata = TRUE, col_regex = "clust", reduced_dim = c("TSNE", "UMAP"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-class.R
\name{TreeIndex}
\alias{TreeIndex}
\title{create a new TreeIndex object}
\usage{
TreeIndex(hierarchy = NULL, feature_order = NULL)
}
\arguments{
\item{hierarchy}{hierarchy as a data.table}

\item{feature_order}{order of the tree if different from colnames}
}
\value{
a `TreeIndex` object
}
\description{
create a new TreeIndex object
}
\examples{
library(metagenomeSeq)
data(mouseData)
counts <- MRcounts(mouseData)
hierarchy <- fData(mouseData)
tree <- TreeIndex(hierarchy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-class.R
\name{.generate_node_ids}
\alias{.generate_node_ids}
\title{generate node ids in the tree}
\usage{
.generate_node_ids(hierarchy_tree, feature_order)
}
\arguments{
\item{hierarchy_tree}{hierarchy as a data.table}

\item{feature_order}{order of the tree if different from colnames}
}
\value{
a data frame object
}
\description{
generate node ids in the tree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/startTreeViz.R
\name{startTreeviz}
\alias{startTreeviz}
\title{Start treeviz app and create \code{\link[scTreeViz]{TreeVizApp}} object to manage connection.}
\usage{
startTreeviz(
  data = NULL,
  genes = NULL,
  top_genes = 100,
  host = "http://epiviz.cbcb.umd.edu/treeviz",
  register_function = .register_all_treeviz_things,
  delay = 2L,
  ...
)
}
\arguments{
\item{data}{TreeViz object to explore}

\item{genes}{(character vector) genes (rownames) to include in heatmap}

\item{top_genes}{(integer) number of top variable genes to include in the heatmap}

\item{host}{(character) host address to launch.}

\item{register_function}{(function) function used to register actions and charts on the treeviz app.}

\item{delay}{(integer) number of seconds to wait for application to load in browser}

\item{...}{additional parameters passed to \code{\link[epivizr]{startEpiviz}}.}
}
\value{
An object of class \code{\link[scTreeViz]{TreeVizApp}}
}
\description{
Start treeviz app and create \code{\link[scTreeViz]{TreeVizApp}} object to manage connection.
}
\examples{
# see package vignette for example usage
app <- startTreeviz(non_interactive=TRUE, open_browser=FALSE)
app$stop_app()

}
\seealso{
\code{\link[scTreeViz]{TreeVizApp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeViz-class.R
\docType{class}
\name{TreeViz-class}
\alias{TreeViz-class}
\title{TreeViz class wrapper for SummarizedExperiment objects}
\description{
TreeViz class wrapper for SummarizedExperiment objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeIndex-class.R
\name{.generate_hierarchy_tree}
\alias{.generate_hierarchy_tree}
\title{generate hierarchy tree}
\usage{
.generate_hierarchy_tree(hierarchy, feature_order)
}
\arguments{
\item{hierarchy}{hierarchy as a data.table}

\item{feature_order}{order of the tree if different from colnames}
}
\value{
a data frame object
}
\description{
generate hierarchy tree
}
